/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.error;

import javax.servlet.http.HttpServletRequest;

/**
 * This abstract class represents the base error that open SUIT
 * can handle in a nice way.
 * That is:
 *   - display localized messages,
 *   - redirect the user to a specific view point after he has acknowledged the error
 */
public abstract class IError extends Exception
{
	/**
     * 
     */
    private static final long serialVersionUID = -8597258490505833587L;
    /**
	 * Error type: functional
	 * A functional error will lead to display a message box styled page
	 */
	public static final short FUNCTIONAL_ERROR = 0;
	/**
	 * Error type: technical
	 * A technical error will lead to display an error box styled page (with complete stack trace).
	 */
	public static final short TECHNICAL_ERROR = 1;
	
	private short type = TECHNICAL_ERROR;
	
	public IError(String message)
	{
		super(message);
	}
	public IError(String message, Throwable cause)
	{
		super(message, cause);
	}
	
	public abstract String getTitle(HttpServletRequest iRequest);

	public abstract String getMessage(HttpServletRequest iRequest);
	
	public abstract String getNextUrl();
	
	public short getType()
    {
    	return type;
    }
	/**
	 * Defines the error type (functional or technical).
	 * A functional error will lead to display a message box styled page,
	 * whereas a technical one will display an error box (with the complete stack trace).
	 * By default, an error is technical.
	 */
	public void setType(short type)
    {
    	this.type = type;
    }

	// la cause est la root exception
	// public abstract String getCause(HttpServletRequest iRequest);

	// Advice n'est pas connu de l'exception (celui qui throw) mais par celui qui catch
	// public abstract String getAdvice(HttpServletRequest iRequest);
}
