/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.error;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xml.base.msg.MessageProvider;


public class LocalizedError extends IError
{
    private static final long serialVersionUID = 1L;
	private String msgProvider;
	private String errorKey;
	private Object[] args;
	private String nextUrl;
	
	public LocalizedError(String iErrorKey, String iMessageProvider, Object[] iArguments, Throwable iRootCause)
	{
		super("LocalizedError ["+iErrorKey+"]", (Throwable)iRootCause);
		
		msgProvider = iMessageProvider;
		errorKey = iErrorKey;
		
		if(iArguments != null){
			args = iArguments.clone();
		}
	}
	public LocalizedError(String iErrorKey, String iMessageProvider, Object[] iArguments)
	{
		this(iErrorKey, iMessageProvider, iArguments, null);
	}
	public LocalizedError(String iErrorKey, Object[] iArguments)
	{
		this(iErrorKey, null, iArguments, null);
	}
	public LocalizedError(String iErrorKey, Object[] iArguments, Throwable iRootCause)
	{
		this(iErrorKey, null, iArguments, iRootCause);
	}
	public LocalizedError(String iErrorKey, String iMessageProvider)
	{
		this(iErrorKey, iMessageProvider, null, null);
	}
	public LocalizedError(String iErrorKey, Throwable iRootCause)
	{
		this(iErrorKey, null, null, iRootCause);
	}
	public LocalizedError(String iErrorKey)
	{
		this(iErrorKey, null, null, null);
	}
	public String getTitle(HttpServletRequest iRequest)
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		MessageProvider provider = OpenSuitSession.getCurrentApplication().getMessageProvider(msgProvider);
		
		if(provider == null){
			return "!"+errorKey+"!";
		}
		
		return provider.getMessage(session.getLocale(), "error."+errorKey+".title", args);
	}
	public String getMessage(HttpServletRequest iRequest)
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		MessageProvider provider = OpenSuitSession.getCurrentApplication().getMessageProvider(msgProvider);
		
		if(provider == null){
			return "!"+errorKey+"!";
		}
		
		return provider.getMessage(session.getLocale(), "error."+errorKey+".message", args);
	}
	/*
	public String getCause(HttpServletRequest iRequest)
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		MessageProvider provider = session.getApplication().getMessageProvider(msgProvider);
		if(provider == null)
			return "!"+errorKey+"!";
		return provider.getMessage(session.getLocale(), "error."+errorKey+".cause", args);
	}
	public String getAdvice(HttpServletRequest iRequest)
	{
		// TODO: valeur par d�faut (si 'error.CODE.text' n'existe pas, ...)
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		MessageProvider provider = session.getApplication().getMessageProvider(msgProvider);
		if(provider == null)
			return "!"+errorKey+"!";
		return provider.getMessage(session.getLocale(), "error."+errorKey+".advice", args);
	}
	*/
	public String getNextUrl()
    {
    	return nextUrl;
    }
	public void setNextUrl(String nextUrl)
    {
    	this.nextUrl = nextUrl;
    }
}
