/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.expression;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation allows to declare that a method is an expression function
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface ExprFunction
{
	/**
	 * Defines the name of the method that compiles this function.<br>
	 * The targeted method must have the following signature:<br>
	 * public void methodName(OpenSuitCompilationContext compilationContext, ICompilationLogger compilationLogger, Method method, List&lt;IExpression&gt; arguments
	 * @return
	 */
	String compileMethod() default "";
	/**
	 * Defines the name of the method that is able to compute the value pattern.<br>
	 * The targeted method must have the following signature:<br>
	 * public String methodName(Method method, List&lt;IExpression&gt; arguments)
	 * @return
	 */
	String valuePatternMethod() default "";
}
