/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.expression;

import java.lang.reflect.Type;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.cel.ICompilationContext;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xml.interfaces.IBeanProvider.UnresolvedBeanError;


/**
 * This is the expression compilation context implementation for open SUIT.
 */
public class OpenSuitCompilationContext implements ICompilationContext
{
	private Application application;
	private IBeanProvider beanProvider;
	
	public OpenSuitCompilationContext(Application application, IBeanProvider beanProvider)
	{
		this.application = application;
		this.beanProvider = beanProvider;
	}
	public Application getApplication()
	{
		return application;
	}
	public Type getVariableGenericType(String iName) throws UnresolvedVariableError
	{
		if ("request".equals(iName)){
			return HttpServletRequest.class;
		}
		
		if ("locale".equals(iName)){
			return Locale.class;
		}
		
		try
		{
			return beanProvider.getBeanGenericType(iName);
		}
		catch (UnresolvedBeanError e)
		{
			throw new UnresolvedVariableError();
		}
	}

	public Class<?> getVariableType(String iName) throws UnresolvedVariableError
	{
		if ("request".equals(iName)){
			return HttpServletRequest.class;
		}
		
		if ("locale".equals(iName)){
			return Locale.class;
		}
		
		try
		{
			return beanProvider.getBeanType(iName);
		}
		catch (UnresolvedBeanError e)
		{
			throw new UnresolvedVariableError();
		}
	}

	public List<IFunctionContext> getFunctions(String prefix, String name)
	{
		return application.getFunctions(prefix, name);
	}
}
