/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.context;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.session.PageContext;
import org.ow2.opensuit.xml.base.Application;


public class OpenSuitSessionImpl extends OpenSuitSession
{
	private static final long serialVersionUID = 1L;
	
	// no: do not serialize
//	private Application application;
	// do not serialize: mark as transient
	private transient ILocaleConfig localeConfig;
	private Locale locale;
	private PageContext context;
	
	public static OpenSuitSessionImpl enterRequest(Application app, HttpServletRequest iRequest, HttpServletResponse iResponse)
	{
		CURRENT_APP.set(app);
		CURRENT_REQ.set(iRequest);
		CURRENT_RESP.set(iResponse);
		OpenSuitSessionImpl session = (OpenSuitSessionImpl)iRequest.getSession().getAttribute("_OPENSUIT_CTX_");
		
		if(session == null)
		{
			// --- create session
			session = new OpenSuitSessionImpl();
			iRequest.getSession().setAttribute("_OPENSUIT_CTX_", session);
		}
		return session;
	}
	public static void exitRequest()
	{
		CURRENT_APP.remove();
		CURRENT_REQ.remove();
		CURRENT_RESP.remove();
	}

	public static OpenSuitSessionImpl getSession(HttpServletRequest iRequest)
	{
		return (OpenSuitSessionImpl)iRequest.getSession().getAttribute("_OPENSUIT_CTX_");
	}
	public void setLocaleConfig(ILocaleConfig localeConfig)
	{
		locale = localeConfig.getLocale();
		this.localeConfig = localeConfig;
	}
	@Override
	public Locale getLocale()
	{
		return locale;
	}
	public ILocaleConfig getLocaleConfig()
    {
		if(localeConfig == null && locale != null){
			// --- session has probably been serialized, localeConfig was lost
			localeConfig = getCurrentApplication().getLocaleConfig(locale);
		}
		
    	return localeConfig;
    }
	
	// ================================================================
	// === Page FormRenderingContext Attributes
	// ================================================================
	private Map<String, Object> name2Attr = new HashMap<String, Object>();
	
	public Object getAttribute(String iName)
	{
	    return name2Attr.get(iName);
	}
	public Set<String> getAttributeNames()
	{
	    return name2Attr.keySet();
	}
	public void removeAttribute(String iName)
	{
		name2Attr.remove(iName);
	}
	public void setAttribute(String iName, Object iValue)
	{
		name2Attr.put(iName, iValue);
	}
	
	
	public PageContext getCurrentPageContext()
	{
		return context;
	}
	public void setCurrentPageContext(PageContext iContext)
	{
		context = iContext;
	}


}
