/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.file.fake_j2ee;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.Principal;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.ow2.opensuit.core.util.UrlBuilder;

// ======================================================================================
// === FakeRequest
// ======================================================================================
public class FakeRequest extends AttributeHolder implements HttpServletRequest {
	private Locale locale = Locale.getDefault();
	private HttpSession session;
	private String characterEncoding = Charset.defaultCharset().name();
	private UrlBuilder url;
	private String method = "GET";
	private String contentType = "application/x-www-form-urlencoded";
	private String protocol = "HTTP/1.1";
	private String scheme = "http";

	public FakeRequest(UrlBuilder url, HttpSession session) {
		this.session = session;
		this.url = url;
	}

	public String getQueryString() {
		String urlStr = url.toUrl(characterEncoding, false);
		int idx = urlStr.indexOf('?');
		return idx < 0 ? null : urlStr.substring(idx+1);
	}

	public String getServletPath() {
		int idxServlet = url.getRequestUri().indexOf('/', 1);
		return url.getRequestUri().substring(idxServlet);
	}

	public String getContextPath() {
		int idxServlet = url.getRequestUri().indexOf('/', 1);
		return url.getRequestUri().substring(0, idxServlet);
	}

	public String getPathInfo() {
		int idxServlet = url.getRequestUri().indexOf('/', 1);
		int idxPath = url.getRequestUri().indexOf('/', idxServlet+1);
		return url.getRequestUri().substring(idxPath);
	}

	public String getRequestURI() {
		return url.getRequestUri();
	}

	public StringBuffer getRequestURL() {
		return new StringBuffer(url.toUrl(characterEncoding, false));
	}

	public String getPathTranslated() {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration<?> getParameterNames() {
		return Collections.enumeration(url.getParameterNames());
	}

	public String getParameter(String iKey) {
		return url.getParameterValue(iKey);
	}

	public Map<?, ?> getParameterMap() {
		return url.getParametersMap();
	}

	public String[] getParameterValues(String arg0) {
		// TODO Auto-generated method stub
		return null;
	}


	
	
	public Locale getLocale() {
		return locale;
	}

	public void setLocale(Locale locale) {
		this.locale = locale;
	}

	public String getMethod() {
		return method;
	}

	public void setMethod(String method) {
		this.method = method;
	}

	public HttpSession getSession() {
		return session;
	}

	public String getCharacterEncoding() {
		return characterEncoding;
	}
	public void setCharacterEncoding(String characterEncoding) throws UnsupportedEncodingException {
		this.characterEncoding = characterEncoding;
	}


	public int getContentLength() {
		// TODO Auto-generated method stub
		return 0;
	}

	public String getContentType() {
		return contentType;
	}

	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	public ServletInputStream getInputStream() throws IOException {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration<?> getLocales() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getProtocol() {
		return protocol;
	}
	public void setProtocol(String protocol) {
		this.protocol = protocol;
	}

	public BufferedReader getReader() throws IOException {
		// TODO Auto-generated method stub
		return null;
	}

	public String getRealPath(String arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public String getRemoteAddr() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getRemoteHost() {
		// TODO Auto-generated method stub
		return null;
	}

	public RequestDispatcher getRequestDispatcher(String arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public String getScheme() {
		return scheme;
	}
	public void setScheme(String scheme) {
		this.scheme = scheme;
	}

	public String getServerName() {
		// TODO Auto-generated method stub
		return null;
	}

	public int getServerPort() {
		// TODO Auto-generated method stub
		return 0;
	}

	public boolean isSecure() {
		// TODO Auto-generated method stub
		return false;
	}

	public String getAuthType() {
		// TODO Auto-generated method stub
		return null;
	}

	public Cookie[] getCookies() {
		// TODO Auto-generated method stub
		return null;
	}

	public long getDateHeader(String arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	public String getHeader(String arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration<?> getHeaderNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration<?> getHeaders(String arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public int getIntHeader(String arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	public String getRemoteUser() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getRequestedSessionId() {
		// TODO Auto-generated method stub
		return null;
	}

	public HttpSession getSession(boolean arg0) {
		return session;
	}

	public Principal getUserPrincipal() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean isRequestedSessionIdFromCookie() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isRequestedSessionIdFromUrl() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isRequestedSessionIdFromURL() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isRequestedSessionIdValid() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isUserInRole(String arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer("FakeRequest[");
		sb.append("Method: ");
		sb.append(getMethod());
		sb.append(", ");
		sb.append("Url: ");
		sb.append(getServletPath());
		sb.append(", ");
		sb.append("Params: ");
		Enumeration<?> names = getParameterNames();
		int nbParams = 0;
		while (names.hasMoreElements()) {
			String name = (String) names.nextElement();
			String value = getParameter(name);

			if (nbParams > 0) {
				sb.append("&");
			}

			sb.append(name);
			sb.append("=");
			sb.append(value);
			nbParams++;
		}

		sb.append("]");
		return sb.toString();
	}

	// ===================================================================
	// === Servlet 2.4 API
	// ===================================================================
	public String getLocalAddr() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getLocalName() {
		// TODO Auto-generated method stub
		return null;
	}

	public int getLocalPort() {
		// TODO Auto-generated method stub
		return 0;
	}

	public int getRemotePort() {
		// TODO Auto-generated method stub
		return 0;
	}
	
}
