/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.file.fake_j2ee;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

public class FakeResponse implements HttpServletResponse
{
	private ServletWriter writer = new ServletWriter();
	private PrintWriter ptWriter;
	private Locale locale;
	private String encoding = Charset.defaultCharset().name();
	
	private int statusCode = 0;
	private String redirectUrl;
	private String contentType;
	
	private int bufferSize = 100;

	
	private Map<String, String> headers = new LinkedHashMap<String, String>();
	
	public FakeResponse()
	{
	}
	public StringBuffer getResponseContent()
	{
		return writer.getBuffer();
	}
	public void addCookie(Cookie cookie)
    {
	    // TODO Auto-generated method stub
	    
    }

	public void addDateHeader(String key, long value)
    {
		headers.put(key, String.valueOf(value));
    }

	public void addHeader(String key, String value)
    {
		headers.put(key, value);
    }

	public void addIntHeader(String key, int value)
    {
		headers.put(key, String.valueOf(value));
    }

	public void setDateHeader(String key, long value)
    {
		headers.put(key, String.valueOf(value));
    }

	public void setHeader(String key, String value)
    {
		headers.put(key, String.valueOf(value));
    }

	public void setIntHeader(String key, int value)
    {
		headers.put(key, String.valueOf(value));
    }
	public boolean containsHeader(String key)
    {
		return headers.containsKey(key);
    }

	public String encodeRedirectUrl(String url)
    {
		try
        {
	        return URLEncoder.encode(url, encoding);
        }
        catch(UnsupportedEncodingException e)
        {
	        return URLEncoder.encode(url);
        }
    }

	public String encodeRedirectURL(String url)
    {
		try
        {
	        return URLEncoder.encode(url, encoding);
        }
        catch(UnsupportedEncodingException e)
        {
	        return URLEncoder.encode(url);
        }
    }

	public String encodeUrl(String url)
    {
		try
        {
	        return URLEncoder.encode(url, encoding);
        }
        catch(UnsupportedEncodingException e)
        {
	        return URLEncoder.encode(url);
        }
    }

	public String encodeURL(String url)
    {
		try
        {
	        return URLEncoder.encode(url, encoding);
        }
        catch(UnsupportedEncodingException e)
        {
	        return URLEncoder.encode(url);
        }
    }

	public void sendError(int sc, String message) throws IOException
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		statusCode = sc;
    }

	public void sendError(int sc) throws IOException
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		statusCode = sc;
    }
	public int getStatusCode() {
		return statusCode;
	}

	public void sendRedirect(String url) throws IOException
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		redirectUrl = url;
    }


	public void setStatus(int sc, String message)
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		statusCode = sc;
    }

	public void setStatus(int sc)
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		statusCode = sc;
    }

	public String getCharacterEncoding()
    {
	    return encoding;
    }

	public Locale getLocale()
    {
	    return locale;
    }

	public ServletOutputStream getOutputStream() throws IOException
    {
	    return new FakeServletOutputStream(getWriter());
    }

	public PrintWriter getWriter() throws IOException
    {
		if(ptWriter == null)
		{
			ptWriter = new PrintWriter(writer);
        }
		return ptWriter;
    }

	public boolean isCommitted()
    {
	    return statusCode != 0 || writer.isFlushed();
    }


	public void flushBuffer() throws IOException
    {
		writer.flush();
    }

	public void reset()
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		writer.getBuffer().setLength(0);
    }

	public void resetBuffer()
    {
		// check state
		if(isCommitted()){
			throw new IllegalStateException("Response already commited");
		}
		
		writer.getBuffer().setLength(0);
    }

	public int getBufferSize()
    {
	    return bufferSize;
    }

	public void setBufferSize(int size)
    {
		bufferSize = size;
    }

	public void setContentLength(int len)
    {
	    // TODO Auto-generated method stub
    }

	public void setContentType(String contentType)
    {
		this.contentType = contentType;
    }
	public String getContentType() {
		return contentType;
	}

	public void setLocale(Locale locale)
    {
		this.locale = locale;
    }
	
	// ===================================================================
	// === Servlet 2.4 API
	// ===================================================================
	public void setCharacterEncoding(String encoding) {
		this.encoding = encoding;
	}

	// ===================================================================
	// === Servlet Output Stream
	// ===================================================================
	private class ServletWriter extends Writer
	{
		private StringBuffer buffer = new StringBuffer();
		private boolean isFlushed = false;
		private boolean isClosed = false;
		
		public StringBuffer getBuffer() {
			return buffer;
		}
		@Override
		public void write(char[] cbuf, int off, int len) throws IOException {
			
			if(isClosed()){
				throw new IllegalStateException("Response already closed");
			}
			
			buffer.append(cbuf, off, len);
		}
		public boolean isClosed() {
			return isClosed;
		}
		public boolean isFlushed() {
			return isFlushed || buffer.length() >= bufferSize;
		}
		@Override
		public void flush() throws IOException {
			isFlushed = true;
		}
		@Override
		public void close() throws IOException {
			isClosed = true;
			statusCode = HttpServletResponse.SC_OK;
		}
		
	}
	// ===================================================================
	// === Servlet Output Stream
	// ===================================================================
	private static class FakeServletOutputStream extends ServletOutputStream
	{
		private PrintWriter writer;
		
		public FakeServletOutputStream(PrintWriter writer) {
			this.writer = writer;
		}

		@Override
        public void print(String arg0) throws IOException
        {
			writer.print(arg0);
        }

		@Override
        public void print(boolean arg0) throws IOException
        {
			writer.print(arg0);
        }

		@Override
        public void print(char arg0) throws IOException
        {
			writer.print(arg0);
        }

		@Override
        public void print(int arg0) throws IOException
        {
			writer.print(arg0);
        }

		@Override
        public void print(long arg0) throws IOException
        {
	        writer.print(arg0);
        }

		@Override
        public void print(float arg0) throws IOException
        {
	        writer.print(arg0);
        }

		@Override
        public void print(double arg0) throws IOException
        {
	        writer.print(arg0);
        }

		@Override
        public void println() throws IOException
        {
	        writer.println();
        }

		@Override
        public void println(String arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void println(boolean arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void println(char arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void println(int arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void println(long arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void println(float arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void println(double arg0) throws IOException
        {
	        writer.println(arg0);
        }

		@Override
        public void close() throws IOException
        {
	        writer.close();
        }

		@Override
        public void flush() throws IOException
        {
	        writer.flush();
        }

		@Override
        public void write(byte[] b, int off, int len) throws IOException
        {
	        // TODO Auto-generated method stub
//???	        writer.write(b, off, len);
        }

		@Override
        public void write(byte[] b) throws IOException
        {
	        // TODO Auto-generated method stub
//???	        writer.write(b);
        }

		@Override
        public void write(int b) throws IOException
        {
	        writer.write(b);
        }
		
	}
}
