/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.j2ee;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.StringTokenizer;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.xml.interfaces.IRequestProcessor;
import org.ow2.opensuit.xml.interfaces.IOpenSuitInstContext;
import org.ow2.opensuit.xmlmap.XmlInstantiator;
import org.ow2.opensuit.xmlmap.impl.IXmlModelMessage;


public class OpenSuitServlet extends HttpServlet
{
	private static Log logger = LogFactory.getLog(OpenSuitServlet.class);
	
//	private static final String PROCESSOR_ATTR_NAME = "OpenSUIT.Processor";
    private static final long serialVersionUID = 1L;

	private String charset = "UTF-8";
	
	private Exception initException;
	private XmlInstantiator xmlInst;
	private IRequestProcessor requestProcessor;
	
	/**
	 * Servlet Initialisation
	 */
	public void init() throws ServletException
	{
		try
		{
			// --- charset
			if(getInitParameter("encoding") != null)
			{
				// --- try charset
				try
				{
					Charset.forName(getInitParameter("encoding"));
					charset = getInitParameter("encoding");
				}
				catch(Exception e)
				{
					logger.error(" --> Warning: encoding charset '"+getInitParameter("encoding")+"' not found.", e);
				}
			}
			
			// --- instanciate the request processor
			String xmlfile = getInitParameter("xml");
			
			if(xmlfile == null){
				throw new ServletException("Init Parameter 'xml' unspecified.");
			}
			
			URL url = getServletContext().getResource(xmlfile);
			
			if(url == null){
				throw new ServletException("xml file '"+xmlfile+"' not found.");
			}
			
			IOpenSuitInstContext validSupport = new J2EEValidator(getServletContext());
			xmlInst = XmlInstantiator.instantiate(url, validSupport);
			
			StringWriter messages = new StringWriter();
			xmlInst.dumpMessages(IXmlModelMessage.VALID_WARNING, new PrintWriter(messages));
			if(messages.getBuffer().length() == 0)
			{
				logger.info("Initialized open SUIT '"+xmlfile+"' successfully.");
			}
			else
			{
				logger.info("Initialized open SUIT '"+xmlfile+"' with messages:");
				logger.info(messages.toString());
			}
			
			if(xmlInst.getRoot() == null)
			{
				logger.error(" --> web site root node instantiated from '"+xmlfile+"' is null.");
			}
			else if (!(xmlInst.getRoot() instanceof IRequestProcessor))
			{
				logger.error(" --> web site root node instantiated from '"+xmlfile+"' is not a IRequestProcessor: "+xmlInst.getRoot().getClass().getName());
			}
			else
			{
				requestProcessor = (IRequestProcessor)xmlInst.getRoot();
//				logger.info(" --> web site root node successfully instantiated from '"+xmlfile+"'.");
			}
		}
		catch(Exception e)
		{
			initException = e;
			logger.error("Error while initializing.", e);
		}
	}
	private String getCharSet()
	{
		return charset;
	}

	/**
	 * main service method
	 */
	public void service(HttpServletRequest iRequest, HttpServletResponse iResponse) throws ServletException, IOException
	{
		// --- display errors if any
		if(initException != null || requestProcessor == null)
		{
			renderInitErrors(iRequest, iResponse);
		}
		else
		{
			// --- process and render request
//			IRequestProcessor processorForUser = (IRequestProcessor)iRequest.getSession().getAttribute(PROCESSOR_ATTR_NAME);
//			if(processorForUser == null)
//			{
//				// --- set current processor
//				processorForUser = requestProcessor;
//				iRequest.getSession().setAttribute(PROCESSOR_ATTR_NAME, processorForUser);
//			}
			try
			{
				requestProcessor.service(iRequest, iResponse);
			}
			catch(IOException ioe)
			{
				throw ioe;
			}
			catch(Throwable t)
			{
				renderStackPage(iRequest, iResponse, t);
			}
		}
//		request.getOutputStream().flush();
	}
	private void renderInitErrors(HttpServletRequest iRequest, HttpServletResponse iResponse) throws IOException
	{
		Writer htmlWriter = new OutputStreamWriter(iResponse.getOutputStream(), getCharSet());
//		PrintWriter htmlWriter;
//		IHtmlWriter htmlWriter = new HtmlWriter(wr);
		
		htmlWriter.write("<html>\n");
		htmlWriter.write("<head>\n");
		htmlWriter.write("<title>Initialization Errors</title>\n");
		htmlWriter.write("<meta http-equiv='content-type' content='text/html;charset="+getCharSet()+"'>\n");
		htmlWriter.write("</head>\n");
		htmlWriter.write("<body>\n");
		
		if(xmlInst.getRoot() == null)
		{
			htmlWriter.write("<h1>Web Site Model not initialized.</h1>\n");
		}
		else if(xmlInst.hasMessages(IXmlModelMessage.VALID_ERROR))
		{
			htmlWriter.write("<h1>Web Site Model has errors.</h1>\n");
			htmlWriter.write("<pre>\n");
			xmlInst.dumpMessages(IXmlModelMessage.VALID_ERROR, new PrintWriter(htmlWriter));
			htmlWriter.write("</pre>\n");
		}
		
		if(initException != null){
			renderStack(iRequest, iResponse, htmlWriter, initException);
		}
		
		htmlWriter.write("</body>");
		htmlWriter.write("</html>");
		htmlWriter.flush();
	}
	private void renderStackPage(HttpServletRequest iRequest, HttpServletResponse iResponse, Throwable iThrowable) throws IOException
	{
//		IHtmlWriter htmlWriter = new HtmlWriter(new OutputStreamWriter(iRequest.getOutputStream(), getCharSet()));
		Writer htmlWriter = new OutputStreamWriter(iResponse.getOutputStream(), getCharSet());
		
		htmlWriter.write("<html>\n");
		htmlWriter.write("<head>\n");
		htmlWriter.write("<title>Error</title>\n");
		htmlWriter.write("<meta http-equiv='content-type' content='text/html;charset="+getCharSet()+"'>\n");
		htmlWriter.write("</head>\n");
		htmlWriter.write("<body bgcolor='#FFFFFF' style='font-family: Verdana' text='#192c8f'>\n" );
		
		renderStack(iRequest, iResponse, htmlWriter, iThrowable);
		
		htmlWriter.write("</body>\n");
		htmlWriter.write("</html>\n");
		htmlWriter.flush();
	}
	private void renderStack(HttpServletRequest iRequest, HttpServletResponse iResponse, Writer writer, Throwable e) throws IOException
	{
		writer.write( "<h3 style='color: #990000;'>"+((e.getMessage()==null)?e.getClass().getName():e.getMessage())+"</h3>\n" );
		writer.write( "Exception type : "+e.getClass().getName()+"<BR>\n" );
		
		// --- dump stack
		StringWriter stackInString= new StringWriter();
		PrintWriter print= new PrintWriter(stackInString);
		e.printStackTrace( print );
		print.close();

		StringTokenizer tokens= new StringTokenizer( stackInString.toString(), "\n" );
		
		if( tokens.hasMoreElements() ){
		    tokens.nextElement();
		}
		
		while( tokens.hasMoreElements() )
		{
			String token= tokens.nextToken().trim();
			if( token.startsWith("at ") )
			{
				token= token.substring(3);
				writer.write( "Exception thrown in :<BR><A style='color: #990000;'>"+token+"</A>\n<P>" );
				break;
			}
		}
		tokens= new StringTokenizer( stackInString.toString(), "\n" );

		writer.write( "Stack trace :<BR><UL>\n" );
		
		if( tokens.hasMoreElements() ){
		    tokens.nextElement();
		}
		
		while( tokens.hasMoreElements() )
		{
			String token= tokens.nextToken().trim();
			
			if( token.startsWith("at ") ){
				token= token.substring(3);
			}else{
				continue;
			}
			
			if( token.indexOf("java.lang.reflect.Method.invoke")>=0 ){
				break;
			}
			
			if( token.indexOf("javax.servlet.http.HttpServlet.service")>=0 ){
				break;
			}
			
			writer.write( "<LI>"+token+"</LI>\n" );
		}
		writer.write( "</UL>\n");

		writer.write( "<A style='text-decoration: underline; cursor: hand;' onClick='stack.style.visibility=\"visible\";'>Show Full Stack</A><BR>\n" );
		writer.write( "<PRE id=stack style='visibility: hidden;'>\n" );
		writer.write( stackInString.toString() );
		writer.write( "\n</PRE>\n" );
		writer.flush();
//		e.printStackTrace();
	}
	// ==================================================================
	// === Properties based messages
	// ==================================================================
	/*
	private Properties loadMessagesFile(String iLanguange)
	{
		try
        {
			String key = iLanguange == null ? "messages" : "messages."+iLanguange;
			String messfile = getInitParameter(key);
			if(messfile == null)
				return null;
			InputStream is = getServletContext().getResourceAsStream(messfile);
			if(is == null)
				return null;
	        Properties ppties = new Properties();
	        ppties.load(is);
	        is.close();
	        return ppties;
        }
        catch(Exception e)
        {
        }
        return null;
	}
	private Properties getMessagesFile(String iLanguange)
	{
		if(iLanguange == null)
			return _defaultMsgs;
		Properties msgs = (Properties)_locale2Messages.get(iLanguange);
		if(msgs == null)
		{
			msgs = loadMessagesFile(iLanguange);
			if(msgs == null)
				msgs = _defaultMsgs;
			_locale2Messages.put(iLanguange, msgs);
		}
		return msgs;
	}
	*/

	// ==================================================================
	// === IRendererSupport implementation
	// ==================================================================
	public static class J2EEValidator implements IOpenSuitInstContext
	{
		private ServletContext ctx;
		
		public J2EEValidator(ServletContext iImageContext)
		{
			ctx = iImageContext;
		}
		public ServletContext getServletContext()
		{
			return ctx;
		}
		public boolean checkResource(String iResourcePath)
		{
			if(ctx == null){
				return true;
			}
			
			if(iResourcePath == null){
			    return true;
			}
			
			if(!iResourcePath.startsWith("/")){
			    iResourcePath = "/"+iResourcePath;
			}
			
//			return ctx.getResourceAsStream(iResourcePath) != null;
            try {
                return ctx.getResource(iResourcePath) != null;
            } catch (MalformedURLException e) {
                return false;
            }
		}
		public Object getAncestor(Class type)
		{
			if(type.isAssignableFrom(ServletContext.class)){
				return ctx;
			}
			
			return null;
		}
	}
	
}
