/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.multiparts;

import java.io.InputStream;

public interface IFileUploadPart extends IPart
{
	/**
	 * Returns the uploading file name. 
	 * @return the uploading file name.
	 */
	String getFileName();
	/**
	 * Returns the uploading file content-type. 
	 * @return the uploading file content-type.
	 */
	String getFileContentType();
	/**
	 * Returns the uploading file input stream. 
	 * @return the uploading file input stream.
	 */
	InputStream getFileInputStream();
	/**
	 * Determines whether if the uploading file looks invalid or not.
	 * <p>
	 * This method checks first if the filename is <code>null</code>, and then if the stream is empty (zero bytes).<br>
	 * Obviously a non-null filename with a non-null stream size is <b>never</b> considered invalid.<br>
	 * <b>Note</b>: A genuine empty file would then appear invalid. This is a limitation of upload client platforms (web browsers).
	 * @return <code>true</code> if the filename is <code>null</code> or if the file is empty (zero bytes).
	 */
	boolean isEmptyStream();
}
