// ===================================================================
// === Open SUIT JavaScript Library
// ===================================================================
// === This library is used by the ChoiceTreeField component
// ===================================================================

/**
 * Anchor node click handler
 * Expands and collapses sub tree items.
 * @param evt source of the event
 */
function ChoiceTree_onClickAnchor(evt)
{
	var anchor = window.event ? event.srcElement : evt.target;
	var nodeLI = getFatherNode("LI", anchor);
	var childrenUL = getFirstChild("UL", nodeLI);
	if(hasClass(nodeLI, "collapsed"))
	{
		// --- expand
		removeClass(nodeLI, "collapsed");
		addClass(nodeLI, "expanded");
	}
	else // expanded
	{
		// --- collapse
		removeClass(nodeLI, "expanded");
		addClass(nodeLI, "collapsed");
	}
}
/**
 * Recursively expands the given node
 */
function ChoiceTree_expandAll(li)
{
	if(hasClass(li, "collapsed"))
	{
		removeClass(li, "collapsed");
		addClass(li, "expanded");
	}
	// --- recurse
	var childrenUL = getFirstChild("UL", li);
	if(childrenUL)
	{
		var items = getChildren("LI", childrenUL);
		for(var i=0; i<items.length; i++)
		{
			if(getFirstChild("UL", items[i]))
				ChoiceTree_expandAll(items[i]);
		}
	}
}
/**
 * Checkbox node click handler
 * Selects/Unselects descendant children items.
 * @param evt source of the event
 */
function ChoiceTree_onClickNode(evt)
{
	var check = window.event ? event.srcElement : evt.target;
	var nodeLI = getFatherNode("LI", check);
	var curState = ChoiceTree_getState(nodeLI);
//	var newState = hasClass(check, "checked") ? false : true;
	var newState = curState == 1 ? false : true;
	// --- set children states (recursively)
	ChoiceTree_setState(nodeLI, newState);
	// --- update parent state
	var parentLI = nodeLI.parentNode.parentNode;
	if(parentLI.nodeName == "LI")
		ChoiceTree_updateState(parentLI);
	
	// --- expand all
	ChoiceTree_expandAll(nodeLI);
}
/**
 * Sets state recursively
 */
function ChoiceTree_setState(li, state)
{
	var input = getFirstChild("INPUT", li);
	if(input)
	{
		// --- this is a leaf
		input.checked = state;
	}
	else
	{
		// --- this is a node
		// --- 1: change checkbox class
		var link = getChildren("A", li)[1];
		link.className = "osuit-ChoiceTreeNode "+(state ? "checked" : "unchecked");
		
		// --- 2: recurse
		var childrenUL = getFirstChild("UL", li);
		if(childrenUL)
		{
			var items = getChildren("LI", childrenUL);
			for(var i=0; i<items.length; i++)
				ChoiceTree_setState(items[i], state);
		}
	}
}
/**
 * Determines the node item state
 *  - Explicit for leaf items (given by the input state)
 *  - Implicit for node items (3 states)
 */
function ChoiceTree_getState(li)
{
	var input = getFirstChild("INPUT", li);
	if(input)
		// --- this is a leaf
		return input.checked ? 1 : 0;

	// --- this is a node
	var childrenUL = getFirstChild("UL", li);
	if(!childrenUL)
		// should not occur
		return 0;
	var selState = -1;// unknown state
	var items = getChildren("LI", childrenUL);
	for(var i=0; i<items.length; i++)
	{
		var childState = ChoiceTree_getState(items[i]);
		switch(selState)
		{
			case -1: // unknown state
				selState = childState;
				break;
			case 0: // so far unselected
				if(childState != 0)
					return 2;// partly
				break;
			case 1: // so far selected
				if(childState != 1)
					return 2;
				break;
		}
	}
	return selState;
}
function ChoiceTree_onClick(evt)
{
	var src = window.event ? event.srcElement : evt.target;
	var li = getFatherNode("LI", src);
	var input = getFirstChild("INPUT", li);
	if(!input)
		return;
	// --- a checkbox state changed
	var parentLI = src.parentNode.parentNode.parentNode;
	if(parentLI.nodeName == "LI")
		ChoiceTree_updateState(parentLI);
}
function ChoiceTree_updateState(li)
{
	var newState = ChoiceTree_getState(li);
	var link = getChildren("A", li)[1];
	var curState = hasClass(link, "unchecked") ? 0 : hasClass(link, "checked") ? 1 : 2;
	if(curState == newState)
		// --- state doesn't change
		return;
	// --- 1: change checkbox class
	link.className = "osuit-ChoiceTreeNode "+(newState == 0 ? "unchecked" : newState == 1 ? "checked" : "partly");
	// --- 2: propagate to parent
	var parentLI = li.parentNode.parentNode;
	if(parentLI.nodeName == "LI")
		ChoiceTree_updateState(parentLI);
}
