/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.session;

import java.io.Serializable;
import java.util.Locale;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.impl.context.OpenSuitSessionImpl;
import org.ow2.opensuit.xml.base.Application;


/**
 * This class gives access to user session in open SUIT.<br>
 * Main notions managed by the open SUIT session are:
 * <ul>
 * <li>current request: the current http request.
 * <li>Application: the root object of the open SUIT application.
 * <li>Locale: the current user locale.
 * <li>Page context: the current page context.
 * </ul>
 */
public abstract class OpenSuitSession implements Serializable
{
	/**
     * 
     */
    private static final long serialVersionUID = 814528154440494525L;
    
    protected static final ThreadLocal<Application> CURRENT_APP = new ThreadLocal<Application>();
	protected static final ThreadLocal<HttpServletRequest> CURRENT_REQ = new ThreadLocal<HttpServletRequest>();
	protected static final ThreadLocal<HttpServletResponse> CURRENT_RESP = new ThreadLocal<HttpServletResponse>();
	
	/**
	 * Returns the current http request.
	 */
	public static HttpServletRequest getCurrentRequest()
	{
		return CURRENT_REQ.get();
	}
	/**
	 * Returns the current http response.
	 */
	public static HttpServletResponse getCurrentResponse()
	{
		return CURRENT_RESP.get();
	}
	/**
	 * Returns the current Open Suit Application. 
	 */
	public static Application getCurrentApplication()
	{
		return CURRENT_APP.get();
	}
	/**
	 * Returns the current Open Suit Session. 
	 */
	public static OpenSuitSession getCurrentSession()
	{
		return OpenSuitSessionImpl.getSession(CURRENT_REQ.get());
	}
	/**
	 * Returns the user's open SUIT session associated to the given http request.
	 * @param iRequest The current http request.
	 */
	public static OpenSuitSession getSession(HttpServletRequest iRequest)
	{
		return OpenSuitSessionImpl.getSession(iRequest);
	}
	/**
	 * Returns the current user's locale.
	 */
	public abstract Locale getLocale();
	/**
	 * Returns the current user's locale configuration.
	 */
	public abstract ILocaleConfig getLocaleConfig();
	
	public abstract void setAttribute(String iName, Object iValue);
	
	public abstract Object getAttribute(String iName);
	
	public abstract void removeAttribute(String iName);
	
	public abstract Set<String> getAttributeNames();
	/**
	 * Returns the current page context.
	 */
	public abstract PageContext getCurrentPageContext();
	
}
