/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import javax.servlet.http.HttpServletRequest;

/**
 * Helper class to manage the current Heading (html elements &lt;Hx&gt;) level 
 * while rendering a page with open SUIT.
 */
public class HeadingUtils
{
	/**
	 * Returns the current heading level during the rendering process.
	 * @param iRequest The current http request.
	 * @return The current heading level.
	 */
	public static int getLevel(HttpServletRequest iRequest)
	{
		Integer level = (Integer)iRequest.getAttribute("_HEADING_LEVEL_");
		
		if(level == null){
			return 1;
		}
		
		return level.intValue();
	}
	/**
	 * Changes the current heading level during the rendering process.
	 * @param iRequest The current http request.
	 * @param level The new heading level to apply.
	 */
	public static void setLevel(HttpServletRequest iRequest, int level)
	{
		iRequest.setAttribute("_HEADING_LEVEL_", Integer.valueOf(level));
	}
	/**
	 * Moves the current heading level by the given increment during the rendering process.
	 * @param iRequest The current http request.
	 * @param increment The increment value to apply to the current heading level.
	 */
	public static void moveLevel(HttpServletRequest iRequest, int increment)
	{
		int level = getLevel(iRequest);
		level+=increment;
		
		if(level <= 0){
			level = 1;
		}
		
		iRequest.setAttribute("_HEADING_LEVEL_", Integer.valueOf(level));
	}
	/**
	 * Increases the current heading level during the rendering process.
	 * @param iRequest The current http request.
	 */
	public static void incrLevel(HttpServletRequest iRequest)
	{
		int level = getLevel(iRequest);
		level++;
		iRequest.setAttribute("_HEADING_LEVEL_", Integer.valueOf(level));
	}
	/**
	 * Decreases the current heading level during the rendering process.
	 * @param iRequest The current http request.
	 */
	public static void decrLevel(HttpServletRequest iRequest)
	{
		int level = getLevel(iRequest);
		level--;
		
		if(level <= 0){
			level = 1;
		}
		
		iRequest.setAttribute("_HEADING_LEVEL_", Integer.valueOf(level));
	}
}
