/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

/**
 * Helper class used by graphical components during the rendering process.
 */
public class RenderingContext
{
	/**
	 * Returns the current rendering context.
	 * @param iRequest The current http request.
	 * @return
	 */
	public static RenderingContext get(HttpServletRequest iRequest)
	{
		RenderingContext ctx = (RenderingContext)iRequest.getAttribute("_ctx_");
		if(ctx == null)
		{
			ctx = new RenderingContext();
			iRequest.setAttribute("_ctx_", ctx);
		}
		return ctx;
	}
	
	private Map<Object, Map<String, Object>> obj2Values = new HashMap<Object, Map<String,Object>>();
	
	/**
	 * Puts data in the rendering context.
	 * @param iSource The component using the data.
	 * @param iName The data name.
	 * @param iValue The value to store.
	 */
	public void put(Object iSource, String iName, Object iValue)
	{
		Map<String, Object> h = (Map<String, Object>)obj2Values.get(iSource);
		if(h == null)
		{
			h = new HashMap<String, Object>();
			obj2Values.put(iSource, h);
		}
		h.put(iName, iValue);
	}
	/**
	 * Retrieves data from the rendering context.
	 * @param iSource The component using the data.
	 * @param iName The data name.
	 * @return The data value.
	 */
	public Object get(Object iSource, String iName)
	{
		Map<String, Object> h = (Map<String, Object>)obj2Values.get(iSource);
		if(h == null){
			return null;
		}
		
		return h.get(iName);
	}
}
