/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.validation;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xml.base.msg.MessageProvider;


public class LocalizedValidationError extends ValidationError
{
	private static final long serialVersionUID = 1L;
	
	private String msgProvider;
	private String key;
	private Object[] args;
	
	public LocalizedValidationError(String iKey, String iMessageProvider, Object[] iArguments)
	{
		msgProvider = iMessageProvider;
		key = iKey;
		
		if(iArguments != null){
			args = iArguments.clone();
		}
	}
	public LocalizedValidationError(String iKey, Object[] iArguments)
	{
		this(iKey, null, iArguments);
	}
	public LocalizedValidationError(String iKey, String iMessageProvider)
	{
		this(iKey, iMessageProvider, null);
	}
	public LocalizedValidationError(String iKey)
	{
		this(iKey, null, null);
	}
	public String getMessage(HttpServletRequest iRequest)
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		MessageProvider provider = OpenSuitSession.getCurrentApplication().getMessageProvider(msgProvider);
		
		if(provider == null){
			return "!"+key+"!";
		}
		
		return provider.getMessage(session.getLocale(), key, args);
	}
}
