/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base;

import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Defines an object that provides expression functions.")
@XmlElement
public class ExpressionFunctionsProvider implements IInitializable
{
	private static final Class<?>[] DEFAULT_CONSTR = {};
	
	@XmlDoc("The functions prefix.")
	@XmlAttribute(name="Prefix",required=false)
	private String prefix;
	
	@XmlDoc("The bean class. Should have a public default constructor if you expect the presentation layer to instantiate it.")
	@XmlAttribute(name="Class")
	private String className;
	
	private Object provider;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		try
        {
			// --- retreive bean class
	        Class<?> c = Class.forName(className);
	        provider = c.newInstance();
        }
        catch(ClassNotFoundException e)
        {
        	initSupport.addValidationMessage(this, "Class", IInitializationSupport.ERROR, "Class '"+className+"' not found.");
        }
        /*
		catch(InvocationTargetException e)
		{
        	initSupport.addValidationMessage(this, "Class", IInitializationSupport.ERROR, "Error occured while instantiating expression functions provider '"+className+"': "+e.getMessage());
		}
		*/
		catch (InstantiationException e)
		{
        	initSupport.addValidationMessage(this, "Class", IInitializationSupport.ERROR, "Error occured while instantiating expression functions provider '"+className+"': "+e.getMessage());
		}
		catch (IllegalAccessException e)
		{
        	initSupport.addValidationMessage(this, "Class", IInitializationSupport.ERROR, "Error occured while instantiating expression functions provider '"+className+"': "+e.getMessage());
		}
	}
	public String getPrefix()
	{
		return prefix;
	}
	public Object getProvider()
	{
		return provider;
	}
	@Override
	public String toString()
	{
		return "ExpressionFunctionProvider ("+className+")";
	}
}
