/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import java.net.MalformedURLException;
import java.net.URL;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A simple hyperlink.<br>" +
		"This action simply navigates to the given url.")
@XmlElement
public class HyperLink implements IAction, IInitializable
{
	private static Log logger = LogFactory.getLog(HyperLink.class);
	
	@XmlDoc("The hyperlink url.")
	@XmlChild(name="Url")
	private Expression url;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check url is valid
		if(url != null && initSupport.initialize(url) && url.isStaticValue())
		{
			try {
				String urlStr = url.invoke(null, String.class);
				try {
					new URL(urlStr);
				} catch (MalformedURLException e) {
					initSupport.addValidationMessage(this, "Url", IInitializationSupport.ERROR, "Malformed url: '"+urlStr+"'");
				}
			} catch (Exception e) {
				// should have not occured as the expression is static
			}
		}
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml)
	{
	    try {
			return url.invoke(iRequest, String.class);
		} catch (Exception e) {
			logger.error("Exception while invoking url:", e);
			return null;
		}
	}
	public void invoke(HttpServletRequest request, HttpServletResponse response) throws Exception
	{
		// --- redirect to the url
		response.sendRedirect(getURL(request, false));
	}
}
