/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlSubstitutionGroup;



/**
 * An action is a piece of logic that handles events, and reacts by navigating
 * to another view.
 * Actions might be either trigger by direct navigation, or invoked by code.
 * @author crhx7117
 */
@XmlDoc("This component interface allows to declare how the application reacts to an event (OnClick, OnError, ...).<br>An Action <b>may</b> trigger application code, and in any cases it <b>must</b> cause a navigation to occur (to a defined view point).")
@XmlSubstitutionGroup
public interface IAction
{
	/**
	 * Returns an url that causes the action to be triggered when navigated.
	 * @param iRequest
	 * @return the action trigger url.
	 * @throws Exception
	 */
	String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception;
	
	/**
	 * Invokes the action and returns the outcome url.
	 * @param iRequest
	 * @return the action outcome url.
	 * @throws Exception
	 */
	void invoke(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception;

}
