/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.Severity;
import org.ow2.opensuit.xml.base.html.CssStyleSheet;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumbDef;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xml.base.param.IUrlParameter;
import org.ow2.opensuit.xml.interfaces.IFrameContent;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This action displays a message, and a button.<br>" +
		"After having read the message, the user should click the button to continue.<br>" +
		"This action is typically used to implement an acknowledgment page.")
@XmlElement
public class Inform implements IAction,  IFrameContent, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("The action ID (for url callbacking).")
	@XmlAttribute(name="ID", required=false)
	private String id = "";
	
	@XmlDoc("The name of the frame this page displays in. May be null (the default one will be used).")
	@XmlAttribute(name="FrameName", required=false)
	private String frameName;
	
    @XmlDoc("The message severity. Default: Info.")
    @XmlAttribute(name="Severity", required=false)
    private Severity severity= Severity.Info;
    
    @XmlDoc("Additional parameters to pass to the action when it is invoked.")
    @XmlChildren(name="AddParameters", minOccurs=0)
    private IUrlParameter[] addParameters;
    
	@XmlDoc("The page displayed title.")
	@XmlChild(name="Title")
	private Expression title;
	
	@XmlDoc("The page breadcrumb trail.")
	@XmlChild(name="BreadCrumb", required=false)
	private IBreadCrumbDef breadcrumb;
	
	@XmlDoc("The displayed message.")
	@XmlChild(name="Message")
	private Expression message;
	
	@XmlDoc("The action to trigger when the user clicks the acknowledgment button.")
	@XmlChild(name="OnOk")
	private IAction onOk;

//	private IUrlParameter[] addParameters;
//	private IBinding _xmlnode_req_Do;
//	private Forward[] _xmlnode_1_unb_Forwards;
//	private IAction _xmlnode_opt_OnContinueAfterError;
	
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/Info_"+id;
		}else{
			return "/"+frame.getName()+"/Info_"+id;
		}
	}
	// --- [-] IIdentifiable implementation
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- register this component as a request handler
		application.registerRequestHandler(getPathID(), this);
//	    if(application.registerRequestHandler(getPathID(), this) != null)
//        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Callback URLs may not be stable on this component. To solve this specify a unique ID in the page.");
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
		// --- create callback url
			// TODO: mettre dans le contexte de la page courante (auto nettoy� lorsqu'on en sort)
//			iRequest.getSession().setAttribute(this.getId()+".args", args);
		
		UrlBuilder url = application.createHandlerUrl(iRequest, this, "invoke");
		if(addParameters != null)
		{
			for(int i=0; i<addParameters.length; i++){
				url.setParameter(addParameters[i].getName(), String.valueOf(addParameters[i].getValue(iRequest)));
			}
		}
		return url.toUrl(iRequest.getCharacterEncoding(), iEncodeHtml);
	}
	/**
	 * Invoke the action: display the ask box page
	 */
	public void invoke(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if("POST".equals(iRequest.getMethod()))
		{
			// --- redirect after post
			iResponse.sendRedirect(getURL(iRequest, false));
			return;
		}
		servePage(iRequest, iResponse);
	}
	// ======================================================================
	// === IFrameContent
	// ======================================================================
	public String getFrameName()
	{
		return frameName;
	}
	public String getTitle(HttpServletRequest request) throws Exception
	{
		return ExpressionUtils.getMessage(title, request);
	}
	public IBreadCrumbDef getBreadCrumbDef()
	{
		return breadcrumb;
	}
	public void servePage(HttpServletRequest iRequest,HttpServletResponse iResponse) throws Exception
	{
		IFrame frame = application.getFrame(getFrameName());
		HtmlUtils.renderPage(iRequest, iResponse, frame, this, this.getTitle(iRequest), 0, frame.getFaviconUrl(iRequest));
	}
	public void preRender(HttpServletRequest request) throws Exception
	{
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		
		iWriter.println("<div class='osuit-Dialog osuit-Inform"+severity.getName()+"'>");
		iWriter.println("<p class='osuit-Message'>");
		iWriter.println(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(message, iRequest)));
		iWriter.println("</p>");
		
		iWriter.print("<div class='osuit-Buttons'>");
		iWriter.print("<ul>");
		
		String nextUrl = onOk.getURL(iRequest, true);
		iWriter.print("<li>");
		iWriter.print("<a class='osuit-Button' href='");
		iWriter.print(nextUrl);
		iWriter.print("'>");
		iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "button.continue")));
		iWriter.print("</a>");
		iWriter.println("</li>");
		
		iWriter.print("</ul>");
		iWriter.println("</div>");

		iWriter.println("</div>");
	}
}
