/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.xml.base.param.IUrlParameter;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This action component causes the client's browser to reload the page (and to discard all popup actions such as Ask and Inform).")
@XmlElement
public class ReloadThisPage implements IAction, IInitializable
{
	@XmlDoc("Overridden parameters to pass to the page when it is invoked. Note that context parameters are automatically kept. There is no need to push them here.")
	@XmlChildren(name="Parameters", minOccurs=0)
	private IUrlParameter[] parameters;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
		OpenSuitUrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
		if(parameters != null)
		{
			for(int i=0; i<parameters.length; i++){
				url.setParameter(parameters[i].getName(), parameters[i].getValue(iRequest));
			}
		}
		return url.toUrl(iRequest.getCharacterEncoding(), iEncodeHtml);
	}
	public void invoke(HttpServletRequest request, HttpServletResponse response) throws Exception
	{
		if("POST".equals(request.getMethod())){
			// --- redirect after post
			response.sendRedirect(getURL(request, false));
		}
	}
}
