/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.binding;

import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.cel.ICompilationMessage;
import org.ow2.opensuit.cel.ICompilationResult;
import org.ow2.opensuit.cel.IExpression;
import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlContent;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("Defines an expression to perform when some application event occurs.")
@XmlElement()
public class Do implements IInitializable
{
	@XmlAncestor
	private Application root;
	
	@XmlAncestor
	private IBeanProvider parentProvider;

	@XmlDoc("The action to execute.")
	@XmlContent
	private String expressionString;
	
	private String id;
	private IExpression expression;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
//		super.initialize(initSupport, additionalSupport);
		
		expressionString = expressionString.trim();
		ICompilationResult compResult = ExpressionUtils.compileExpression(expressionString, root, parentProvider);
		if(compResult.getMessages() != null)
		{
			for(ICompilationMessage message : compResult.getMessages())
			{
				int level = IInitializationSupport.ERROR;
				
				if(message.getLevel() == ICompilationMessage.WARNING_LEVEL){
					level = IInitializationSupport.WARNING;
				}else if(message.getLevel() == ICompilationMessage.INFO_LEVEL){
					level = IInitializationSupport.INFO;
				}
				
				initSupport.addValidationMessage(this, "content", level, message.getMessage()+": \r\n"+(expressionString.substring(0, message.getPosition())));
			}
		}
		expression = compResult.getExpression();
		
		// --- check type
		// TODO
	}
	/*
	private static String makeCharArrow(int position)
	{
		StringBuffer sb = new StringBuffer();
		for(int i=0; i<position-1; i++)
			sb.append('-');
		sb.append('^');
		return sb.toString();
	}
	*/
	public Type getGenericType()
	{
		if(expression == null){
			return null;
		}
		
	    return expression.getGenericType();
	}
	public Class<?> getType()
	{
		if(expression == null){
			return null;
		}
		
	    return expression.getType();
	}
	public boolean isConvertible(Class<?> type)
	{
		if(expression == null){
			return false;
		}
		
		return ExpressionUtils.getTypeConverter().isConvertible(expression.getType(), type);
	}

	public boolean hasErrors()
	{
	    return expression == null;
	}
	public Object invoke(HttpServletRequest iRequest) throws Exception
	{
		if(expression == null){
			return null;
		}
		
		return ExpressionUtils.invokeExpression(expression, root, parentProvider, iRequest);
	}
	public <T> T invoke(HttpServletRequest iRequest, Class<T> type) throws Exception
	{
		if(expression == null){
			return null;
		}
		
		Object ret = ExpressionUtils.invokeExpression(expression, root, parentProvider, iRequest);
		return ExpressionUtils.getTypeConverter().convert(ret, type);
	}
	public String getID()
	{
		if(id == null){
			id = HtmlUtils.formatId(expressionString);
		}
		
	    return id;
	}
	public boolean isStaticValue()
	{
		if(expression == null){
			return true;
		}
		
	    return expression.isStaticValue();
	}
	public boolean isReadOnly()
	{
		if(expression == null){
			return true;
		}
		
	    return expression.isReadOnly();
	}
	public void set(HttpServletRequest iRequest, Object iValue) throws Exception
	{
		if(expression == null){
			return;
		}
		
		ExpressionUtils.setExpression(expression, root, parentProvider, iRequest, iValue);
	}
	@Override
	public String toString()
	{
		return expressionString;
	}
}
