/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.binding;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This allows to declare standard Java beans with their scope.")
@XmlElement
public class JavaBeans implements IDataSource, IInitializable
{
	// --- data sources (beans)
	@XmlDoc("Java beans declarations.")
	@XmlChildren(direct=true, name="Beans")
	private Bean[] beans;
	
	private Map<String, Bean> name2Bean;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check beans don't have the same name
		name2Bean = new HashMap<String, Bean>();
		if(beans != null)
		{
			for(int i=0; i<beans.length; i++)
			{
				// --- trigger initialization
				if(!initSupport.initialize(beans[i])){
					continue;
				}
				
				if(beans[i].getName() == null){
					continue;
				}
				
				if(name2Bean.get(beans[i].getName()) != null)
				{
					initSupport.addValidationMessage(beans[i], "Beans", IInitializationSupport.ERROR, "More than one bean with name '"+beans[i].getName()+"'.");
				}
				else
				{
					name2Bean.put(beans[i].getName(), beans[i]);
				}
			}
		}
	}
	public Class<?> getBeanType(String name) throws UnresolvedBeanError
	{
		Bean bean = name2Bean.get(name);
		
		if(bean == null){
			return null;
		}
		
		return bean.getType();
	}
	public Type getBeanGenericType(String name) throws UnresolvedBeanError
	{
		Bean bean = name2Bean.get(name);
		
		if(bean == null){
			return null;
		}
		
		return bean.getGenericType();
	}
	public Object getBeanValue(HttpServletRequest request, String name) throws Exception
	{
		Bean bean = name2Bean.get(name);
		
		if(bean == null){
			return null;
		}
		
		return bean.getObject(request);
	}
	public List<String> getBeanNames() {
		return new ArrayList<String>(name2Bean.keySet());
	}
	public String getBeanDescription(String name) {
		Bean bean = name2Bean.get(name);
		
		if(bean == null){
			return null;
		}
		try {
			return "User bean ("+bean.getScope()+")<br>" +
					"Class: "+bean.getType().getName();
		} catch (UnresolvedBeanError e) {
			return "User bean ("+bean.getScope()+")<br>" +
			"Class: unresolved!";
		}
	}

}
