/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.enums;

import java.text.DateFormat;

import org.ow2.opensuit.core.session.IDateOutputFormatConfig;
import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlEnumeration;



@XmlDoc("This enumeration allows to set the type of display for a Date.")
@XmlEnumeration
public enum DateType
{
	@XmlDoc("Displays the date only (with format according to the user locale).")
	DateOnly(true, false),
	
	@XmlDoc("Displays the time only (with format according to the user locale).")
	TimeOnly(false,true),
	
	@XmlDoc("Displays the date and the time (with format according to the user locale).")
	DateAndTime(true,true);
	
	private final boolean date;
	private final boolean time;
	private DateType(boolean date, boolean time) {
		this.date = date;
		this.time = time;
	}
	public boolean isDate() {
		return date;
	}
	public boolean isTime() {
		return time;
	}
	public DateFormat getInputFormat(ILocaleConfig config)
	{
		if(this == DateAndTime){
			return config.getDateTimeInputFormat();
		}
		
		if(this == TimeOnly){
			return config.getTimeInputFormat();
		}
		
		return config.getDateInputFormat();
	}
	public DateFormat getOutputFormat(IDateOutputFormatConfig config)
	{
		if(this == DateAndTime){
			return config.getDateTimeOutputFormat();
		}
		
		if(this == TimeOnly){
			return config.getTimeOutputFormat();
		}
		
		return config.getDateOutputFormat();
	}
}
