/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.enums;

import java.util.regex.Pattern;

import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlEnumeration;

/**
 * This enumeration allows to set specified zip code for ZipCode validator
 * 
 * @author Adrien Ruffie / Pierre Smeyers
 * @author http://opensuit.ow2.org/
 * @version 1.0
 */
@XmlDoc("This enumeration allows to set specified zip code for "
		+ "ZipCode validator.")
@XmlEnumeration
public enum ZipCountry {

	/*
	 * #US postal code
	 */
	@XmlDoc("USA: US postal code")
	USA("USA", Pattern.compile("^\\d{5}\\p{Punct}?\\s?(?:\\d{4})?$")),

	/*
	 * #Japanese postal code
	 */
	@XmlDoc("JAP: Japanese postal code")
	JAP("JAP", Pattern.compile("^\\d{3}-\\d{4}$")),

	/*
	 * #Dutch postal code
	 */
	@XmlDoc("DEU: Dutch postal code")
	DEU("DEU", Pattern.compile("^[0-9]{4}\\s*[a-zA-Z]{2}$")),

	/*
	 * #Argentinean postal code
	 */
	@XmlDoc("ARG: Argentinean postal code")
	ARG("ARG", Pattern.compile("^\\d{3}-\\d{4}$")),

	/*
	 * #Swedish postal code
	 */
	@XmlDoc("SWE: Swedish postal code")
	SWE("SWE", Pattern.compile("^(s-|S-){0,1}[0-9]{3}\\s?[0-9]{2}$")),

	/*
	 * #French postal code
	 */
	@XmlDoc("FRA: French postal code")
	FRA("FRA", Pattern.compile("^(2[ab]|0[1-9]|[1-9][0-9])[0-9]{3}$")),

	/*
	 * #Canadian postal code
	 */
	@XmlDoc("CAN: Canadian postal code")
	CAN("CAN", Pattern.compile("^([A-Z]\\d[A-Z]\\s\\d[A-Z]\\d)$")),

	/*
	 * #United Kingdom postal code
	 */
	@XmlDoc("UK: United Kingdom postal code")
	UK("UK", Pattern.compile("^[a-zA-Z]{1,2}[0-9][0-9A-Za-z]{0,1} {0,1}[0-9][A-Za-z]{2}$")),

	/*
	 * #Australian postal code -- Non-Matches(0300 | 7612 | 2915) -- Matches
	 * (0200 | 7312 | 2415)
	 */
	@XmlDoc("AUS: Australian postal code")
	AUS("AUS", Pattern.compile("^(0[289][0-9]{2})|([1345689][0-9]{3})|(2[0-8][0-9]{2})|(290[0-9])|(291[0-4])|(7[0-4][0-9]{2})|(7[8-9][0-9]{2})$ "));

	// Referenced country
	private final String code;

	// Pattern country
	private final Pattern pattern;

	/**
	 * @return the code
	 */
	public String getCode() {
		return this.code;
	}

	/**
	 * @return the pattern
	 */
	public Pattern getPattern() {
		return this.pattern;
	}

	/**
	 * @param code
	 * @param pattern
	 */
	private ZipCountry(final String code, final Pattern pattern) {
		this.code = code;
		this.pattern = pattern;
	}
}
