/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.error;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.interfaces.IFrameContent;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This abstract component allows to declare how to display uncatched errors to the user.<br>This component should know how to best display details, explainations and advises to the user to let him understand what went wrong, and what to do to solve it.")
@XmlElement
public abstract class ErrorHandler implements IFrameContent, IInitializable
{
	protected static Log logger = LogFactory.getLog(ErrorHandler.class);
	@XmlDoc("Defines the name of the Application Frame this page displays in.")
	@XmlAttribute(name="FrameName", required=false)
	protected String frameName;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    // TODO: check frame exists
	}
	public String getFrameName()
	{
	    return frameName;
	}
	public static void setNextUrl(HttpServletRequest iRequest, String nextUrl)
	{
		iRequest.setAttribute("javax.servlet.error.next_url", nextUrl);
	}
	public static String getNextUrl(HttpServletRequest iRequest)
	{
		return (String)iRequest.getAttribute("javax.servlet.error.next_url");
	}
	public static Throwable getError(HttpServletRequest iRequest)
	{
//		return (Throwable)iRequest.getSession().getAttribute("_ERROR_");
		return (Throwable)iRequest.getAttribute("javax.servlet.error.exception");
	}
	public abstract String getTitle(HttpServletRequest iRequest) throws Exception;
	
	public void servePage(HttpServletRequest iRequest, HttpServletResponse iResponse) throws IOException
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		try
		{
			IFrame frame = OpenSuitSession.getCurrentApplication().getFrame(getFrameName());
			HtmlUtils.renderPage(iRequest, iResponse, frame, this, this.getTitle(iRequest), 0, frame.getFaviconUrl(iRequest));
		}
		catch(Throwable t)
		{
			// --- if an error occurs while rendering the error page, display a simple stack
			logger.error("Error while rendering error page:", t);
			
			if(iResponse.isCommitted()){
				logger.error("The request has already been committed.");
			}else{
				iResponse.resetBuffer();
			}
			
			renderStack(session.getLocaleConfig(), iRequest, iResponse, t);
		}
	}
	private void renderStack(ILocaleConfig iLocaleConfig, HttpServletRequest iRequest, HttpServletResponse iResponse, Throwable t) throws IOException
	{
		iResponse.setContentType("text/html;charset="+iLocaleConfig.getCharSet());
		iResponse.setHeader("Cache-Control", "no-store"); //HTTP 1.1
		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server

		PrintWriter htmlWriter = iResponse.getWriter();
		
		// --- development context: display the stack in the page
		htmlWriter.println(HtmlUtils.getHTML401TransitionalDoctype());
		htmlWriter.print("<html lang=\"");
		htmlWriter.print(iLocaleConfig.getLocale().getLanguage());
		htmlWriter.println("\">");
		htmlWriter.println("<head>");
		htmlWriter.println("<title>Error</title>");
		htmlWriter.println("<meta http-equiv='content-type' content='text/html;charset="+iLocaleConfig.getCharSet()+"'>");
		// no cache (for all browsers)
		htmlWriter.println("<meta http-equiv='cache-control' content='no-cache'>");
		htmlWriter.println("<meta http-equiv='pragma' content='no-cache'>");
		//TODO: expires -1 to get reloading the page?
		htmlWriter.println("<meta http-equiv='expires' content='0'>");
		htmlWriter.println("</head>");
		htmlWriter.println("<body bgcolor='#FFFFFF' style='font-family: Verdana; color: #192c8f'>" );
		HtmlUtils.writeStack(htmlWriter, t);
		htmlWriter.println("</body>");
		htmlWriter.println("</html>");
//			htmlWriter.flush(new OutputStreamWriter(iResponse.getOutputStream(), _xmlattr_opt_CharSet));
		htmlWriter.flush();
		htmlWriter.close();
	}
}
