/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;


import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.html.breadcrumb.BreadCrumbElement;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumb;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumbDef;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumb.IBreadCrumbContext;
import org.ow2.opensuit.xml.interfaces.IFrameContent;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;



@XmlDoc("The BreadCrumbContainer component may only be used as a descendant of the Frame component.<br>" +
		"It is aimed at rendering the current page's breadcrumb trail during the rendering phase.")
@XmlElement
public class BreadCrumbContainer implements IView
{
	@XmlDoc("The breadcrumb trail separator sign.<br>" +
			"Default: none (done in CSS).")
	@XmlAttribute(name="Separator", required=false)
	private String separator = null;
	
	@XmlDoc("Determines whether first separator should be displayed.<br>" +
			"Default: true.")
	@XmlAttribute(name="DisplayFirstSeparator", required=false)
	private boolean displayFirst = true;
	
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		IFrameContent page = (IFrameContent)iRequest.getAttribute("_RENDER_PAGE_");
		// --- render breadcrumb trail
		IBreadCrumbDef bcd = page.getBreadCrumbDef();
		IBreadCrumb bc = bcd == null ? null : bcd.getBreadCrumb();
		if(bc != null)
		{
			Context ctx = new Context();
			bc.populate(iRequest, ctx);
			if(ctx.elements.size() > 0)
			{
				PrintWriter iWriter = iResponse.getWriter();
				String sepHtml = separator == null ? null : HtmlUtils.encode2HTML(separator);
				
				iWriter.print("<div class='osuit-BreadCrumb'>");
				iWriter.print("<ul>");
				for(int i=0; i<ctx.elements.size(); i++)
				{
					BreadCrumbElement elt = ctx.elements.get(i);
					
					boolean isFirst = i == 0;
					boolean isLast = i == ctx.elements.size()-1;
					
					// --- separator
					if(sepHtml != null && (displayFirst || !isFirst))
					{
						iWriter.print("<li class=separator>");
						iWriter.print(sepHtml);
						iWriter.println("</li>");
					}
					
					// --- render element
					String url = isLast ? null /* last element is always disabled */ : elt.getUrl();
					iWriter.print("<li>");
					if(url == null)
					{
						iWriter.print("<span");
					}
					else
					{
						iWriter.print("<a");
						// --- render the href
						iWriter.print(" href='");
						iWriter.print(url);
						iWriter.print("'");
					}
					// --- class
					String cssClass = "";
					if(elt.getCssClass() != null)
					{
						cssClass+=" "+elt.getCssClass();
					}
					if(isFirst)
					{
						cssClass += " first";
					}
					else if(isLast)
					{
						cssClass += " last";
					}
					if(cssClass.length() > 1)
					{
						iWriter.print(" class='");
						iWriter.print(cssClass.substring(1));
						iWriter.print("'");
					}
					// --- tooltip
					if(elt.getTooltip() != null)
					{
						iWriter.print(" title='");
					    iWriter.print(HtmlUtils.encode2HTML(elt.getTooltip()));
						iWriter.print("'");
					}
					iWriter.print(">");
					// --- icon
					if(elt.getIconSrc() != null)
					{
						iWriter.print("<img src='");
						iWriter.print(elt.getIconSrc());
						iWriter.print("' alt='");
						
						if(elt.getTooltip() != null){
							iWriter.print(HtmlUtils.encode2HTML(elt.getTooltip()));
						}
						
						iWriter.print("'>");
					}
					// --- title
				    iWriter.print(HtmlUtils.encode2HTML(elt.getTitle()));
					// --- close
				    
				    if(url == null){
				    	iWriter.print("</span>");
				    }else{
				    	iWriter.print("</a>");
				    }
				    
					iWriter.println("</li>");
				}
				iWriter.print("</ul>");
				iWriter.print("</div>");
			}
		}
	}
	private static class Context implements IBreadCrumbContext
	{
		private List<BreadCrumbElement> elements = new ArrayList<BreadCrumbElement>();
		
		public void addElement(BreadCrumbElement element)
		{
			elements.add(element);
		}
	}
}
