/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xml.interfaces.IOpenSuitInstContext;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Includes a link to a stylesheet file (css).")
@XmlElement
public class CssStyleSheet implements IInitializable
{
	@XmlDoc("Path to the stylesheet file.<br>" +
			"The path is relative to the war root directory " +
			"(ex: 'resources/css/style.css').")
	@XmlAttribute(name="File")
	private String file;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(file != null)
		{
			if(file.startsWith("/")){
				file = file.substring(1);
			}
			
			if(!((IOpenSuitInstContext)instContext).checkResource(file)){
				initSupport.addValidationMessage(this, "File", IInitializationSupport.ERROR, "CSS file '"+file+"' not found.");
			}
		}
	}
	/**
	 * Returns the absolute path to the stylesheet
	 * @param iRequest
	 * @return
	 */
	public String getHref(HttpServletRequest iRequest)
	{
		if(file.startsWith("/")){
			return iRequest.getContextPath()+file;
		} else{
			return iRequest.getContextPath()+"/"+file;
		}
	}
}
