/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;


import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Declares an Application Frame.<br>" +
		"<p>Each page of the Application is displayed in a Frame. " +
		"A Frame is the part of the GUI that doesn't change much from page to page. " +
		"It is generally composed of a banner, some menus (left and/or right), and a footer. <br>" +
		"It must embed a PageContainer component, that is in charge of rendering the current executing page (see PageContainer).<br> " +
		"The Application must have at least one Frame, but it may have several (for instance, a specific frame without any menu to display messages and uncatched errors, and another one to access the application administration functionnalities).")
@XmlElement
public class Frame implements IFrame, IInitializable
{
	@XmlAncestor
	private Application application;

	@XmlDoc("The Frame name. Must be unique in the Application.")
	@XmlAttribute(name="Name")
	private String name;
	
	@XmlDoc("The page's favicon.<br>" +
			"Overrides the containing frame's favicon - if any.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/favicon.png').")
	@XmlChild(name="Favicon", required=false)
	private Expression favicon;

	@XmlDoc("List of stylesheet files to use when rendering pages using this frame.")
	@XmlChildren(name="StyleSheets")
	private CssStyleSheet[] styleSheets;
	
	@XmlDoc("The children HTML view components for this Frame. Must have a PageContainer component in its descendents.")
	@XmlChildren(direct=true, name="Views")
	private IView[] views;
	
	// --- [+] IIdentifiable implementation
	public String getPathID()
	{
		return "/Frame_"+name;
	}

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    // TODO Auto-generated method stub
	    // v�rifier qu'il y a au moins un page container ?
		// --- check favicon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Favicon", favicon);
	}
	public CssStyleSheet[] getStyleSheets()
	{
		return styleSheets;
	}
	public String getName()
	{
	    return name;
	}
	public String getFaviconUrl(HttpServletRequest iRequest)
	{
		return favicon == null ? application.getFaviconUrl(iRequest) : ExpressionUtils.getImageSrc(favicon, iRequest);
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		for(int i=0; i<views.length; i++){
			views[i].preRender(iRequest);
		}
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter htmlWriter = iResponse.getWriter();
		// --- open DIV
		htmlWriter.print("<div class='osuit-Frame'");
		htmlWriter.print(" id='");
		htmlWriter.print(name);
		htmlWriter.print("'");
		htmlWriter.println(">");
		
		// --- render content
		for(int i=0; i<views.length; i++){
			views[i].render(iRequest, iResponse);
		}
		
		// --- close DIV
		htmlWriter.println("</div>");
	}
}
