/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;


import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This component allows to insert html code directly in the page rendering.")
@XmlElement
public class HTML implements IView, IInitializable
{
	@XmlDoc("Defines the increment in heading (&lt;Hn&gt;) to apply after having rendered this.")
	@XmlAttribute(name="HeadingIncrement", required=false)
	private int headingIncrement= 0;
	
	@XmlDoc("Defines whether the content should be html encoded. Default: no..")
	@XmlAttribute(name="Encode", required=false)
	private boolean encode = false;
	
	@XmlDoc("The html code.")
	@XmlChild(direct=false, name="Code")
	private Expression code;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		String text = ExpressionUtils.getMessage(code, iRequest);
		
		if(encode){
			iWriter.print(HtmlUtils.encode2HTML(text));
		} else{
			iWriter.print(text);
		}
		
		if(headingIncrement != 0){
			HeadingUtils.moveLevel(iRequest, headingIncrement);
		}
	}
}
