/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html;


import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumb;
import org.ow2.opensuit.xml.base.html.breadcrumb.IBreadCrumbDef;
import org.ow2.opensuit.xml.interfaces.IFrameContent;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("The PageContainer component may only be used as a descendant of the Frame component.<br>" +
		"It is aimed at rendering the current page during the rendering phase.")
@XmlElement
public class PageContainer implements IView, IInitializable
{
//	private Application _xmlroot;
	@XmlDoc("Determines whether this component renders the page title (as an H2 element) before rendering the page.")
	@XmlAttribute(name="RenderTitle", required=false)
	private boolean renderTitle= true;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    // TODO Auto-generated method stub
	    
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		IFrameContent page = (IFrameContent)iRequest.getAttribute("_RENDER_PAGE_");
		page.preRender(iRequest);
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		IFrameContent page = (IFrameContent)iRequest.getAttribute("_RENDER_PAGE_");
		iWriter.println("<div id='osuit-PageContainer'>");
		// --- render page title
		if(renderTitle)
		{
			// --- H1
			iWriter.print("<h1 class='osuit-PageTitle'>");
			iWriter.print(HtmlUtils.encode2HTML(page.getTitle(iRequest)));
			iWriter.print("</h1>");
			HeadingUtils.setLevel(iRequest, 2);
		}
		// --- have the current page render itself
		page.render(iRequest, iResponse);
		
		if(renderTitle){
			HeadingUtils.setLevel(iRequest, 1);
		}
		
		iWriter.println("</div>");
	}
}
