/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HeadingUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A form item that presents a structured sub-section of the form.")
@XmlElement
public class Section implements IFormItem, IInitializable
{
	@XmlAncestor
	private BaseForm baseForm;
	
	@XmlDoc("The displayed section title.")
	@XmlChild(name="Title")
	private Expression title;
	
	@XmlDoc("Visible details (TODO).")
	@XmlChild(name="Visible", required=false)
	private Expression visible;
	
	@XmlDoc("Children section items.")
	@XmlChildren(direct=true, name="Items")
	private IFormItem[] items;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);
	}
	public boolean isVisible(HttpServletRequest iRequest)
	{
		return visible == null || ExpressionUtils.getBoolean(visible, iRequest);
	}
	protected BaseForm getForm()
	{
		return baseForm;
	}
	public void submit(HttpServletRequest iRequest, FormSubmitContext iSubmitContext) throws Exception
	{
		if(!isVisible(iRequest)){
			return;
		}
		
		for(int i=0; i<items.length; i++){
			items[i].submit(iRequest, iSubmitContext);
		}
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(!isVisible(iRequest)){
			return;
		}
		
		for(int i=0; i<items.length; i++){
			items[i].preRender(iRequest);
		}
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(!isVisible(iRequest)){
			return;
		}
		
		FormRenderingContext iContext = getForm().getRenderingContext(iRequest);
		
		PrintWriter iWriter = iResponse.getWriter();
		iWriter.println("<fieldset>");
		/*
		 * Note: il est interdit au niveau HTML 4.01 Transitional
		 * de mettre un <Hn> dans un <legend>
		 * 
		 * Par ailleurs � l'usage on se rend compte que le <legend> est moins utile 
		 * que <Hn> (navigation par titres beaucoup plus utile pour un non-voyant)
		 * 
		 * En cons�quence: on n'utilise plus <legend> mais <Hn>
		 */
//		iWriter.print("<legend>");
		// --- put an heading element
		int hLevel = HeadingUtils.getLevel(iRequest);
		HeadingUtils.incrLevel(iRequest);
		iWriter.print("<h");
		iWriter.print(String.valueOf(hLevel));
		iWriter.print(" class='osuit-Title'>");

		iWriter.print(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(title, iRequest)));

		iWriter.print("</h");
		iWriter.print(String.valueOf(hLevel));
		iWriter.print(">");

//		iWriter.println("</legend>");
		for(int i=0; i<items.length; i++)
		{
			iContext.oddField = i%2 == 0;
			items[i].render(iRequest, iResponse);
		}
		// --- decrement heading level
		HeadingUtils.decrLevel(iRequest);
		iWriter.println("</fieldset>");
	}
}
