/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.html.menu.IMenuItem;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;


@XmlDoc("This component declares a view only form.<br> It is only aimed at presenting non-editable (and navigable) data.")
@XmlElement
public class ViewForm extends BaseForm implements IInitializable
{
	@XmlDoc("Action to trigger when the reload button is clicked.<br> If not set, the reload button is not displayed.")
	@XmlChildren(name="OnReload", minOccurs=0)
	private Do[] onReload;
	@XmlDoc("Additional buttons to display.")
	@XmlChildren(name="AddButtons", minOccurs=0)
	private IMenuItem[] addButtons;

	public void reload(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(onReload != null)
		{
			for(int i=0; i<onReload.length; i++){
				onReload[i].invoke(iRequest);
			}
		}
	}
	@Override
	public boolean isEditable(HttpServletRequest iRequest)
	{
	    return false;
	}
	@Override
	public boolean isEditing(HttpServletRequest iRequest)
	{
	    return false;
	}
	@Override
	public boolean hasAutoFocus(HttpServletRequest iRequest) {
		return false;
	}
	@Override
	public void renderButtons(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		boolean hasButtons = false;
		if(onReload != null && onReload.length > 0)
		{
			hasButtons = true;
		}
		else
		{
			if(addButtons != null)
			{
				hasButtons = true;
				/* TODO: count buttons
				for(int i=0; i<addButtons.length; i++)
				{
					if(addButtons[i].isVisible(iRequest))
					{
						hasButtons = true;
						break;
					}
				}
				*/
			}
		}
		
		if(hasButtons)
		{
			PrintWriter iWriter = iResponse.getWriter();
			iWriter.print("<div class='osuit-Buttons'>");
			iWriter.print("<ul>");
			if(onReload != null && onReload.length > 0)
			{
				String reloadUrl = application.createHandlerUrl(iRequest, this, "reload").toUrl(iRequest.getCharacterEncoding(), true);
				iWriter.print("<li>");
				iWriter.print("<a class='osuit-Button' href='");
				iWriter.print(reloadUrl);
				iWriter.print("'>");
				iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "button.reload")));
				iWriter.print("</a>");
				iWriter.println("</li>");
			}
			if(addButtons != null)
			{
				for(int i=0; i<addButtons.length; i++)
				{
					addButtons[i].render(iRequest, iResponse, null);
				}
			}

			iWriter.print("</ul>");
			iWriter.println("</div>");
		}
	}
	@Override
	public void renderGlobalErrors(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
	}
}
