/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.xml.base.html.form.BaseForm;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.base.html.form.IFormItem;
import org.ow2.opensuit.xml.base.html.table.Table;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("A non-editable form field that displays a table.")
@XmlElement
public class TableField implements IFormItem, IInitializable
{
	@XmlAncestor
	private BaseForm baseForm;
	
	@XmlDoc("The field name. Must be unique in the Form.")
	@XmlAttribute(name="Name")
	protected String name;
	
	@XmlDoc("The table component.")
	@XmlChild(direct=true, name="Table")
	protected Table table;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	}
	protected BaseForm getForm()
	{
		return baseForm;
	}
	public String getFieldName()
	{
		return name;
	}
	public void submit(HttpServletRequest iRequest, FormSubmitContext iSubmitContext) throws Exception
	{
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		table.preRender(iRequest);
	}
	
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		FormRenderingContext iContext = getForm().getRenderingContext(iRequest);
		iWriter.print("<div class='osuit-Field");
		
		if(iContext.isOddField()){
		    iWriter.print(" odd");
		} else{
            iWriter.print(" even");
		}
		
		iWriter.println("'>");
		
		table.render(iRequest, iResponse);
		
		iWriter.println("</div>");
	}
}
