/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.item;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This component allows to define a list of static items.")
@XmlElement
public class InlineItems implements IItemsProvider, IInitializable
{
	@XmlDoc("The displayable items.")
	@XmlChildren(direct=true, name="Items")
	private Item[] items;
	
	private Class<?> itemValueClass;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(items != null && items.length > 0)
		{
			// TODO: pas top :-(
			if(initSupport.initialize(items[0].value)){
				itemValueClass = items[0].value.getType();
			}
		}
	}
	public DisplayableItem[] getItems(HttpServletRequest iRequest) throws Exception
	{
		DisplayableItem[] dItems = new DisplayableItem[items.length];
		for(int i=0; i<dItems.length; i++)
		{
			dItems[i] = new DisplayableItem(items[i].value.invoke(iRequest), ExpressionUtils.getMessage(items[i].text, iRequest));
			
			if(items[i].disabled != null){
				dItems[i].setEnabled(!ExpressionUtils.getBoolean(items[i].disabled, iRequest));
			}
		}
		return dItems;
	}
	public Class<?> getItemValueType()
	{
	    return itemValueClass;
	}
}
