/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.item;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This component defines a static item.")
@XmlElement
public class Item implements IInitializable
{
	@XmlDoc("The item value.")
	@XmlChild(name="Value")
	protected Expression value;
	
	@XmlDoc("The item text.")
	@XmlChild(name="Text")
	protected Expression text;
	
	@XmlDoc("Determines whether this item is disabled.")
	@XmlChild(name="Disabled", required=false)
	protected Expression disabled;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check disabled can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Disabled", disabled);
	}
}
