/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.layout;


import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Renders an html DIV component.<br>" +
		"This is mainly used to implement layout and/or conditional visibility.")
@XmlElement
public class Div implements IView, IInitializable
{
	@XmlDoc("The DIV id attribute.")
	@XmlAttribute(name="ID", required=false)
	private String iD;
	
	@XmlDoc("The DIV class attribute.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass;
	
	@XmlDoc("Visible.")
	@XmlChild(name="Visible", required=false)
	private Expression visible;
	
	@XmlDoc("Children view components.")
	@XmlChildren(direct=true, name="Views")
	private IView[] views;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);
	}
	public boolean isVisible(HttpServletRequest iRequest)
	{
		if(visible == null){
			return true;
		}
		
		return ExpressionUtils.getBoolean(visible, iRequest);
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(!isVisible(iRequest)){
			return;
		}
		
		for(int i=0; i<views.length; i++){
			views[i].preRender(iRequest);
		}
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(!isVisible(iRequest)){
			return;
		}
		
		PrintWriter iWriter = iResponse.getWriter();
		// --- open DIV
		iWriter.print("<div");
		if(iD != null)
		{
			iWriter.print(" id='");
			iWriter.print(iD);
			iWriter.print("'");
		}
		if(cssClass != null)
		{
			iWriter.print(" class='");
			iWriter.print(cssClass);
			iWriter.print("'");
		}
		iWriter.println(">");
		
		// --- render content
		for(int i=0; i<views.length; i++){
			views[i].render(iRequest, iResponse);
		}
		
		// --- close DIV
		iWriter.println("</div>");
	}
}
