/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.tab;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.RenderingContext;
import org.ow2.opensuit.xml.base.html.IView;
import org.ow2.opensuit.xml.base.html.tab.TabPageRenderingContext.TabDef;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;




@XmlDoc("A graphical component that displays children in selectable tabs.")
@XmlElement
public class TabPage implements IView, IInitializable
{
	@XmlAncestor
	private IPage iPage;
	
	@XmlDoc("The component ID. Must be unique in its Page.")
	@XmlAttribute(name="ID")
	private String id;
	
	@XmlDoc("The CSS class to apply. Default: 'osuit-TabPage'.")
	@XmlAttribute(name="CssClass", required=false)
	private String cssClass= "osuit-TabPage";
	
	@XmlDoc("Children tabs.")
	@XmlChildren(direct=true, name="Tabs")
	private ITabsProvider[] tabs;
	
	/*
	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	private IFrame frame;
	public String getPathID()
	{
		if(page != null)
			return "/"+page.getName()+"/Tabs_"+id;
		else
			return "/"+frame.getName()+"/Tabs_"+id;
	}
	// --- [-] IIdentifiable implementation
	 */

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    // TODO Auto-generated method stub
	    
	}
	/*
	private int getTabIndex(String iID)
	{
		if(tabs == null)
			return -1;
		for(int i=0; i<tabs.length; i++)
		{
			if(iID.equals(tabs[i].getID()))
				return i;
		}
		return -1;
	}
	*/
	public String getId()
	{
	    return id;
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		// --- prepare tabpage rendering context
		// --- tab param name starts with underscore for a frame tabpage
		String tabParamName = iPage == null ? "_tab_"+getId() : "tab_"+getId();
		TabPageRenderingContext context = new TabPageRenderingContext(tabParamName);
		for(int i=0; i<tabs.length; i++)
		{
			tabs[i].declareTabs(iRequest, context);
		}
		// --- find selected tab
		TabDef selectedTab = null;
		String reqTabId = iRequest.getParameter(tabParamName);
		if(reqTabId == null && iPage == null)
		{
			// --- for a frame tabpage, retrieve selected tab from session
			reqTabId = (String)OpenSuitSession.getSession(iRequest).getAttribute(tabParamName);
		}
		if(reqTabId != null)
		{
			selectedTab = context.getTab(reqTabId);
			
			if(!selectedTab.isVisible() || !selectedTab.isEnabled()){
				selectedTab = null;
			}
		}
		if(selectedTab == null)
		{
			// --- no tab specified: show the first available
			selectedTab = context.getFirstDisplayableTab();
		}
		
		// --- pre render selected tab
		if(selectedTab != null)
		{
			// --- for a frame tabpage, store selected tab in session
			if(iPage == null)
			{
				OpenSuitSession.getSession(iRequest).setAttribute(tabParamName, selectedTab.getId());
			}

			// --- put rendering context in request
//			iRequest.setAttribute("tabctx_"+getID(), context);
			context.setSelectedTab(selectedTab);
			selectedTab.getProvider().preRender(iRequest, context);
		}
		RenderingContext.get(iRequest).put(this, "context", context);
	}
	/*
	public TabPageRenderingContext getRenderingContext(HttpServletRequest iRequest)
	{
		return (TabPageRenderingContext)iRequest.getAttribute("tabctx_"+getID());
	}
	*/
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		TabPageRenderingContext context = (TabPageRenderingContext)RenderingContext.get(iRequest).get(this, "context");
		
		if(!context.hasVisibleTabs()){
			return;
		}
		
		PrintWriter iWriter = iResponse.getWriter();
		iWriter.println("<div id='"+getId()+"' class='"+cssClass+"'>");
		iWriter.print("<div class='osuit-TabBar'>");
		iWriter.println("<ul>");
		for(int i=0; i<tabs.length; i++)
		{
			tabs[i].renderTabs(iRequest, iResponse, context);
		}
		iWriter.print("</ul>");
		iWriter.println("</div>");
		
		// --- render content
		if(context.getSelectedTab() != null)
		{
			iWriter.println("<div class='osuit-TabContent'>");
			context.getSelectedTab().getProvider().renderContent(iRequest, iResponse, context);
			iWriter.println("</div>");
		}
		iWriter.println("</div>");
	}
}
