/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlSubstitutionGroup;



@XmlDoc("This interface represents a component that is able to render a Table column.")
@XmlSubstitutionGroup
public interface IColumnsProvider
{
	void declareColumns(HttpServletRequest iRequest, TableRenderingContext iContext) throws Exception;

	void renderHeader(HttpServletRequest iRequest, HttpServletResponse iResponse, TableRenderingContext context, ColumnDef column, int icol, ILinkHrefRenderer iLinkRenderer) throws Exception;

	void renderCell(HttpServletRequest iRequest, HttpServletResponse iResponse, TableRenderingContext context, ColumnDef column, int icol, Object iRowObject) throws Exception;
	
	String getColumnTitle(HttpServletRequest iRequest, ColumnDef column) throws Exception;
	/**
	 * Returns the cell displayed text
	 * Used by some IFilter to retrieve the cell message
	 * @param iRequest
	 * @param iRowObject
	 * @return
	 * @throws Exception
	 */
	Object getCellDisplayedContent(HttpServletRequest iRequest, ColumnDef column, Object iRowObject) throws Exception;
	
	/**
	 * Returns the cell internal value (or display text if none)
	 * Used by IFilter and ISorter to get the internal cell value
	 * @param iRequest
	 * @param iRowObject
	 * @return
	 * @throws Exception
	 */
	Object getCellValue(HttpServletRequest iRequest, ColumnDef column, Object iRowObject) throws Exception;
	
	Class<?> getCellValueType();
	
	Type getCellValueGenericType();
}
