/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.table.ITableRenderingContext;
import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xml.interfaces.ILinkHrefRenderer;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlSubstitutionGroup;



@XmlDoc("This interface represents a component that is able to filter table rows.")
@XmlSubstitutionGroup
public interface IFilter
{
	void render(HttpServletRequest iRequest, HttpServletResponse iResponse, ITableRenderingContext context, ILinkHrefRenderer iLinkRenderer, int icol) throws Exception;
	
	/**
	 * Computes an intermediate object (any) before processing filtering.
	 * @param iRequest http request
	 * @param iFilter the input text filter value (non-null, non-empty)
	 * @return
	 * @throws Exception
	 */
	Object compileFilter(HttpServletRequest iRequest, String iFilter) throws Exception;

	/**
	 * Tests whether the value passes the filter.
	 * @param iRequest http request
	 * @param iCompiledFilter compiled filter object
	 * @param text text value to test
	 * @return <code>true</code> if the value passes the test, <code>false</code> otherwise.
	 * @throws Exception
	 */
	boolean passFilter(HttpServletRequest iRequest, Object iCompiledFilter, ColumnDef column, Object iRowObject) throws Exception;
}
