/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xml.base.html.table.TableRenderingContext.ColumnDef;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Component that performs rows sort based on the Comparable interface.")
@XmlElement
public class NativeSorter implements ISorter, IInitializable
{
	@XmlAncestor
	private IColumnsProvider column;
	
	@XmlDoc("Sets whether the sort is case unsensisitive (for String objects only). Default: <code>true</code>.")
	@XmlAttribute(name="IgnoreCase", required=false)
	private boolean ignoreCase= true;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		Class<?> cellType = column.getCellValueType();
    	if(cellType != null && !cellType.isPrimitive() && !Comparable.class.isAssignableFrom(cellType)){
    		initSupport.addValidationMessage(this, null, IInitializationSupport.ERROR, "Sortable value has to implement Comparable: "+cellType.getName());
    	}
	}
	public Object getSortableValue(HttpServletRequest iRequest, ColumnDef column, Object iRowObject) throws Exception
	{
		return this.column.getCellValue(iRequest, column, iRowObject);
	}
	public int compare(HttpServletRequest iRequest, Object val1, Object val2) throws Exception
	{
		if(val1 == null)
		{
			if(val2 == null){
				return 0;
			}
			return 1;
		}
		else
		{
			if(val2 == null){
				return -1;
			}
			
			// --- val1 & val2 not null
			if(ignoreCase && val1 instanceof String){
				return ((String)val1).compareToIgnoreCase(String.valueOf(val2));
			}
			
			if(val1 instanceof Comparable<?>){
				return ((Comparable)val1).compareTo(val2);
			}	else{
				return 0;
			}
		}
	}
}
