/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.ReflectionHelper;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("SelectionBindings details (TODO).")
@XmlElement
public class SelectionBindings implements IInitializable
{
	@XmlDoc("Binding that retrieves the row ID (use of contextual row bean).")
	@XmlChild(name="RowID")
	protected Expression rowID;
	
	@XmlDoc("Binding that gets/sets the selected rows (vector data with type compatible with the RowID binding).")
	@XmlChild(name="SelectedIDs")
	protected Expression selectedIDs;
	
	private Class<?> idType;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(initSupport.initialize(rowID))
		{
			idType = rowID.getType();
			if(String.class != idType && !ExpressionUtils.getTypeConverter().isConvertible(idType, Integer.class)){
				initSupport.addValidationMessage(this, "RowID", IInitializationSupport.ERROR, "Expression 'RowID' has invalid type: String, Integer or int expected.");
			}
		}
		if(initSupport.initialize(selectedIDs))
		{
			Class<?> selectedIdType = ReflectionHelper.getVectorElementClass(selectedIDs.getGenericType());
			if(selectedIdType == null){
				initSupport.addValidationMessage(this, "SelectedIDs", IInitializationSupport.ERROR, "Expression 'SelectedIDs' must be vector type.");
			} else	{
			    
				if(String.class != selectedIdType && !ExpressionUtils.getTypeConverter().isConvertible(selectedIdType, Integer.class)){
					initSupport.addValidationMessage(this, "SelectedIDs", IInitializationSupport.ERROR, "Expression 'SelectedIDs' has invalid element type: String, Integer or int expected.");
				} else if(idType != null && !idType.equals(selectedIdType)) {
					initSupport.addValidationMessage(this, "SelectedIDs", IInitializationSupport.ERROR, "Expression 'SelectedIDs' element type ("+selectedIdType+") incompatible with 'RowID' type ("+idType+").");
				}
				if(selectedIDs.isReadOnly()){
					initSupport.addValidationMessage(this, "SelectedIDs", IInitializationSupport.ERROR, "Expression 'SelectedIDs' must be settable.");
				}
			}
		}
		/*
		if(_xmlnode_opt_SelectAll != null && !_xmlnode_opt_SelectAll.hasErrors())
		{
			Class selectAllType = _xmlnode_opt_SelectAll.getType();
			if(Boolean.class != selectAllType && boolean.class != selectAllType)
				iErrors.logMessage(this, "SelectAll", IValidityLogger.ERROR, "SelectAll binding must be of type Boolean or boolean.");
			if(!_xmlnode_opt_SelectAll.isGetSet())
				iErrors.logMessage(this, "SelectAll", IValidityLogger.ERROR, "SelectAll binding cannot be set.");
		}
		*/
	}
	public Class<?> getIDType()
	{
		return idType;
	}
}
