/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table.export;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.action.OpenFilePolicy;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.table.Table;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This action may only be used in a Table toolbar. Triggering this action exports table data.")
@XmlElement
public class ExportTableData implements IAction, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("The policy the browser should apply to open the downloaded file." +
			"<ul>" +
			"<li><strong>AUTO</strong>: let the browser manage itself what to do (default),</li>" +
			"<li><strong>SAVE</strong>: the browser should popup the \"save as\" dialog box,</li>" +
			"<li><strong>INLINE</strong>: the browser should display the document inline.</li>" +
			"</ul>")
	@XmlAttribute(name="OpenPolicy",required=false)
	private OpenFilePolicy openPolicy = OpenFilePolicy.AUTO;
	
	@XmlDoc("Export format.")
	@XmlChild(name="Format")
	private ITableExportFormat format;

	@XmlAncestor
	private Table table;

	// --- [+] IIdentifiable implementation
	@XmlAncestor
	private IPage page;
	@XmlAncestor
	private IFrame frame;
	public String getPathID()
	{
		if(page != null){
			return page.getPath()+"/"+table.getId()+"/Export";
		} else{
			return "/"+frame.getName()+"/"+table.getId()+"/Export";
		}
	}
	// --- [-] IIdentifiable implementation
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    if(table == null)
	    {
        	initSupport.addValidationMessage(this, null, IInitializationSupport.ERROR, "May only be used in a Table toolbar.");
	    }
	    else
	    {
			// --- register this component as a request handler
			application.registerRequestHandler(getPathID(), this);
	    }
	}
	public String getURL(HttpServletRequest iRequest, boolean iEncodeHtml) throws Exception
	{
//		UrlBuilder url = application.createHandlerUrl(iRequest, this, "invoke");
		UrlBuilder url = application.createServiceUrl(iRequest, this, "invoke", format.getFileName());
		return url.toUrl(iRequest.getCharacterEncoding(), iEncodeHtml);
	}
	/**
	 * Invoke the action: display the ask box page
	 */
	public void invoke(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		if(openPolicy != OpenFilePolicy.AUTO)
		{
			String fileNameStr = format.getFileName();
			if(fileNameStr != null)
			{
				int idx = fileNameStr.lastIndexOf('/');
				
				if(idx >= 0){
					fileNameStr = fileNameStr.substring(idx+1);
				}
				
				// --- this header forces the browser behavior
				// content-disposition: attachment|inline
				iResponse.setHeader("Content-Disposition", (openPolicy == OpenFilePolicy.INLINE ? "inline" : "attachment")+"; filename="+fileNameStr);
			}
		}
		
		table.export(iRequest, iResponse, format);
	}
}
