/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.table.filter;

import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;

@XmlDoc("A filter function that filters values containing the entered value.")
@XmlElement
public class Contains implements IFilterFunction
{
	@XmlDoc("Sets whether the filter is case unsensitive. Default: <code>true</code>.")
	@XmlAttribute(name="IgnoreCase", required=false)
	private boolean ignoreCase= true;
	
	@XmlDoc("Sets the filter policy if more than one word is entered. Default: <code>ALL</code>.")
	@XmlAttribute(name="Policy", required=false)
	private ContainsPolicy policy= ContainsPolicy.ALL;
	
	public Object compileFilter(HttpServletRequest request, String filter) throws Exception
	{
// not useful
		filter = filter.trim();
		
		if(ignoreCase){
			filter = filter.toLowerCase();
		}
		
//		String[] words = filter.split(" +");
		StringTokenizer st = new StringTokenizer(filter, " ,;");
		String[] words = new String[st.countTokens()];
		
		for(int i=0; i<words.length; i++){
			words[i] = st.nextToken();
		}
		
		return words;
	}
	public boolean passFilter(HttpServletRequest request, Object compiledFilter, String cellText) throws Exception
	{
		if(cellText == null){
			return false;
		}
		
		String[] words = (String[])compiledFilter;
		
		if(ignoreCase){
			cellText = cellText.toLowerCase();
		}
		
		for(String word : words)
		{
			if(cellText.indexOf(word) >= 0)
			{
				// --- cell contains word
				if(policy == ContainsPolicy.ANY){
					return true;
				}
			}
			else
			{
				// --- cell does not contain word
				if(policy == ContainsPolicy.ALL){
					return false;
				}
			}
		}
		// --- policy ALL: return true; policy ANY: return FALSE
		return (policy == ContainsPolicy.ALL);
	}
}
