/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.tree;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;

@XmlDoc("Declares a tree leaf.")
@XmlElement
public class Leaf implements ITreeLeaf, IInitializable, IInlineItem
{
	@XmlDoc("Leaf text.")
	@XmlChild(name="Text")
	private Expression text;
	
	@XmlDoc("Leaf tooltip.")
	@XmlChild(name="Tooltip",required=false)
	private Expression tooltip;
	
	@XmlDoc("Leaf icon.<br>" +
			"The path returned by this expression is relative to the war " +
			"root directory (ex: 'resources/images/icon.png').<br>" +
			"Supported contextual beans: $item.")
	@XmlChild(name="Icon", required=false)
	private Expression icon;
	
	@XmlDoc("Action to trigger when the tree item is clicked.<br>" +
		"Supported contextual beans: $item.")
	@XmlChild(name="OnClick", required=false)
	private IAction onClick;
	
	@XmlDoc("Leaf visibility.<br>" +
			"Expected return type: boolean.")
	@XmlChild(name="Visible", required=false)
	private Expression visible;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- check visible can be converted to a boolean
		ExpressionUtils.validateBooleanExpr(initSupport, instContext, this, "Visible", visible);

		// --- check icon expression
		ExpressionUtils.validateImageSrc(initSupport, instContext, this, "Icon", icon);
	}
	public boolean isVisible(HttpServletRequest request)
	{
		if(visible == null){
			return true;
		}
		
		return ExpressionUtils.getBoolean(visible, request);
	}
	public String getIcon(HttpServletRequest request)
	{
		if(icon == null){
			return null;
		}
		
		return ExpressionUtils.getImageSrc(icon, request);
	}
	public String getText(HttpServletRequest request)
	{
		return ExpressionUtils.getMessage(text, request);
	}
	public String getTooltip(HttpServletRequest request)
	{
		return tooltip == null ? null : ExpressionUtils.getMessage(tooltip, request);
	}
	public String getUrl(HttpServletRequest request, boolean encodeHtml) throws Exception
	{
		return onClick == null ? null : onClick.getURL(request, encodeHtml);
	}
}
