/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.msg;

import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;

import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;



@XmlDoc("A MessageProvider is a component able to provide localized messages from a key and a given locale.")
@XmlElement
public abstract class MessageProvider implements IInitializable
{
	@XmlDoc("The provider name. Must be unique in the Application.")
	@XmlAttribute(name="Name")
	private String name;
	
	public String getName()
	{
		return name;
	}
	
	public abstract boolean checkMessage(Locale iLocale, String iKey);
	
	public abstract List<String> getMatchingKeys(Locale iLocale, Pattern iKeyPattern, int max);

	public abstract String getMessage(Locale iLocale, String iKey, Object[] iArgs);
}
