/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.page;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.action.IAction;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.enums.Severity;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlDoc("A MessageBox page.<br>Such a page displays a message, and an acknowledgement button.<br>The acknowledgement button shall bring the user to another page.")
@XmlElement
public class MessageBox extends BasePage implements IInitializable
{
	@XmlDoc("The message severity. Default: Info.")
	@XmlAttribute(name="Severity", required=false)
	private Severity severity= Severity.Info;
	
	@XmlDoc("The displayed message.")
	@XmlChild(name="Message")
	private Expression message;
	
	@XmlDoc("The action to trigger when the user clicks the acknowledgement button.")
	@XmlChild(name="OnOk")
	private IAction onOk;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
	}
	public void render(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		
		iWriter.println("<div class='osuit-Dialog osuit-Inform"+severity.getName()+"'>");
		iWriter.println("<p class='osuit-Message'>");
		iWriter.println(HtmlUtils.encode2HTML(ExpressionUtils.getMessage(message, iRequest)));
		iWriter.println("</p>");
		
		iWriter.print("<div class='osuit-Buttons'>");
		iWriter.print("<ul>");
		
		String nextUrl = onOk.getURL(iRequest, true);
		iWriter.print("<li>");
		iWriter.print("<a class='osuit-Button' href='");
		iWriter.print(nextUrl);
		iWriter.print("'>");
		iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "button.continue")));
		iWriter.print("</a>");
		iWriter.println("</li>");
		
		iWriter.print("</ul>");
		iWriter.println("</div>");

		iWriter.println("</div>");
	}
}
