/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.param;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Defines a dynamic url parameter.")
@XmlElement
public class DynParam implements IUrlParameter, IInitializable
{
	@XmlDoc("Url parameter name.")
	@XmlAttribute(name="Name")
	private String name;
	
	@XmlDoc("Url parameter value.")
	@XmlChild(direct=false, name="Value")
	private Expression value;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- warning in case parameter name starts with '_'
		if(name != null && name.startsWith("_"))
			initSupport.addValidationMessage(this, "Name", IInitializationSupport.WARNING, "Careful: a parameter name starting with '_' has a specific behavior (not transfered from one request to another).");
		
	    // --- check binding is of serializable type ?
		if(value != null && initSupport.initialize(value))
		{
			Class t = value.getType();
			if(t == null || t == Void.class)
			{
				initSupport.addValidationMessage(this, "Value", IInitializationSupport.ERROR, "Value expression must return something.");
			}
			else
			{
				if(t.isPrimitive())
				{}
				else if(t == String.class)
				{}
				else if(t == Boolean.class)
				{}
				else if(Number.class.isAssignableFrom(t))
				{}
				else if(Enum.class.isAssignableFrom(t))
				{}
				else
				{
					// --- warning
					initSupport.addValidationMessage(this, "Value", IInitializationSupport.INFO, "Value expression is not a simple type; it may be tough to read.");
				}
			}
		}
	}
	public String getName()
	{
		return name;
	}
	public String getValue(HttpServletRequest iRequest) throws Exception
	{
		Object ret = value.invoke(iRequest);
		
		if(ret == null) {
			return null;
		}
		
		return String.valueOf(ret);
	}
}
