/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.param;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.util.OpenSuitUrlBuilder;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Passes the current viewpoint (page and arguments) as a navigation parameter.")
@XmlElement
public class ViewPointParam implements IUrlParameter, IInitializable
{
	@XmlDoc("The name of the navigation parameter that will contain the viewpoint.")
	@XmlAttribute(name="Name")
	private String name;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		// --- warning in case parameter name starts with '_'
		if(name != null && name.startsWith("_"))
			initSupport.addValidationMessage(this, "Name", IInitializationSupport.WARNING, "Careful: a parameter name starting with '_' has a specific behavior (not transfered from one request to another).");
	}
	public String getName()
	{
		return name;
	}
	public String getValue(HttpServletRequest iRequest) throws Exception
	{
		OpenSuitUrlBuilder url = OpenSuitUrlBuilder.copyFromRequest(iRequest, true);
		return url.toUrl(iRequest.getCharacterEncoding(), false);
	}
}
