/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.validation;

import java.lang.reflect.Type;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.field.IValidable;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Validator component that calls a method from the code to perform validation.")
@XmlElement
public class CustomValidator extends BaseBeanProvider implements IValidator, IInitializable
{
	private static Log logger = LogFactory.getLog(CustomValidator.class);
	
	@XmlAncestor
	private IValidable iValidable;
	
	@XmlDoc("The name of the value contextual bean.<br>" + 
			"This bean contains the value that is being validated.<br>" + 
			"Default: $value.")
	@XmlAttribute(name="ValueBeanName", required=false)
	private String valueBeanName= "$value";
	
	@XmlDoc("The method that performs validation.<br>" +
			"This method has to throw a ValidationError to complain about validation issue(s).")
	@XmlChild(direct=false, name="Validate")
	private Expression validate;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
	}
	public boolean isTypeValid(Class<?> iClass)
	{
		// --- type validation done in expression compilation
		return true;
	}
	public void checkValidity(Object iValue) throws ValidationError
	{
		if(iValue == null){
			// --- allowed
			return;
		}
		
		HttpServletRequest request = OpenSuitSession.getCurrentRequest();
		request.setAttribute(valueBeanName, iValue);
		try
		{
			validate.invoke(request);
		}
		catch (ValidationError e)
		{
			throw e;
		}
		catch (Exception e)
		{
			logger.error("Exception while invoking custom validator.", e);
		}
	}
	public Type getBeanGenericType(String name) throws UnresolvedBeanError
	{
		if(name.equals(valueBeanName)){
			return iValidable.getValueGenericType();
		}
		
		return getParentBeanGenericType(name);
	}
	public Class<?> getBeanType(String name) throws UnresolvedBeanError
	{
		if(name.equals(valueBeanName)){
			return iValidable.getValueType();
		}
		
		return getParentBeanType(name);
	}
	public Object getBeanValue(HttpServletRequest request, String name) throws Exception
	{
		if(name.equals(valueBeanName)){
			return request.getAttribute(name);
		}
		
		return getParentBeanValue(request, name);
	}
	public List<String> getBeanNames() {
		List<String> names = getParentBeanNames();
		names.add(valueBeanName);
		return names;
	}
	public String getBeanDescription(String iName) {
		if(valueBeanName.equals(iName))
		{
			return "The &lt;CustomValidator&gt; value contextual bean.<br>" + 
			"This bean contains the value that is being validated.<br>" + 
			"Class: "+iValidable.getValueType().getName();
		}
		return getParentBeanDescription(iName);
	}

}
