/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.validation;

import java.util.regex.Pattern;

import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Validator that checks that the input is recognized as an email.")
@XmlElement
public class Email implements IValidator, IInitializable
{
	private static final Pattern EMAIL_RE = Pattern.compile("[a-zA-Z0-9_\\-\\.]+@[a-zA-Z0-9-]{2,}([.][a-zA-Z]{2,3})+");

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
	    // TODO Auto-generated method stub
	    
	}
	public boolean isTypeValid(Class<?> iClass)
	{
	    return iClass == String.class;
	}
	public void checkValidity(Object iValue) throws ValidationError
	{
		if(iValue == null){
			// --- allowed
			return;
		}

		if(iValue instanceof String)
		{
			if(((String)iValue).length() == 0){
				// --- same as null value
				return;
			}
			
			if(!EMAIL_RE.matcher((String)iValue).matches())
			{
				throw new LocalizedValidationError("validator.email.not_an_email", "opensuit");
			}
		}
	}
}
