/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.validation;

import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("For an integer value, checks it is greater than the Min and/or lower than the Max.<br>For a String value, checks its length is greater than the Min and/or lower than the Max.<br>")
@XmlElement
public class MinMax implements IValidator, IInitializable
{
	@XmlDoc("The minimum value (or length).")
	@XmlAttribute(name="Min", required=false)
	private long min= Long.MIN_VALUE;
	@XmlDoc("The maximum value (or length).")
	@XmlAttribute(name="Max", required=false)
	private long max= Long.MAX_VALUE;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		if(min >= max)
		{
			initSupport.addValidationMessage(this, null, IInitializationSupport.ERROR, "Min value must be lower than Max value.");
		}
	}
	public boolean isTypeValid(Class<?> iClass)
	{
	    return String.class == iClass || Number.class.isAssignableFrom(iClass) || iClass == Short.TYPE || iClass == Integer.TYPE || iClass == Long.TYPE;
	}
	public void checkValidity(Object iValue) throws ValidationError
	{
		if(iValue == null){
			// --- allowed
			return;
		}

		Class<?> c = iValue.getClass();
		if(Number.class.isAssignableFrom(c) || c == Short.TYPE || c == Integer.TYPE || c == Long.TYPE)
		{
			long i = ((Number)iValue).longValue();
			if(i < min)
			{
				throw new LocalizedValidationError("validator.min_max.int_too_low", "opensuit", new Object[]{min});
			}
			else if(i > max)
			{
				throw new LocalizedValidationError("validator.min_max.int_too_high", "opensuit", new Object[]{max});
			}
		}
		else if(iValue instanceof String)
		{
			String s = (String)iValue;
			int i = s.length();
			if(i < min)
			{
				throw new LocalizedValidationError("validator.min_max.str_too_low", "opensuit", new Object[]{min});
			}
			else if(i > max)
			{
				throw new LocalizedValidationError("validator.min_max.str_too_high", "opensuit", new Object[]{max});
			}
		}
		// else must be detect during validation
	}
}
