/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.validation;

import java.util.regex.Pattern;

import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("Checks the associated value (String) matches the specified regular expression.")
@XmlElement
public class RegExp implements IValidator, IInitializable
{
	@XmlDoc("The regular expression patternStr.")
	@XmlAttribute(name="Pattern")
	private String patternStr;
	
	@XmlDoc("The error message to display if the value does not match the expression.")
	@XmlChild(name="ErrorMessage", required=false)
	private Expression errorMessage;
	
	private Pattern pattern;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		pattern = Pattern.compile(patternStr);
	}
	public boolean isTypeValid(Class<?> iClass)
	{
	    return iClass == String.class;
	}
	public void checkValidity(Object iValue) throws ValidationError
	{
		if(iValue == null){
			// --- allowed
			return;
		}

		if(iValue instanceof String)
		{
			String v = ((String)iValue).trim();
			
			if(v.length() == 0){
				// --- allowed
				return;
			}
			
			if(!pattern.matcher(v).matches())
			{
				if(errorMessage == null){
					throw new LocalizedValidationError("validator.regexp.doesnt_match", "opensuit");
				} else{
					throw new BaseValidationError(errorMessage);
				}
			}
		}
	}
}
