/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.interfaces;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;


@XmlElement
public abstract class BaseBeanProvider implements IBeanProvider, IInitializable
{
	@XmlAncestor
	private IBeanProvider parentProvider;
	
	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
//System.out.println("bean provider ancestor of "+this+": "+parentProvider);
		// --- require parent initialization
		// No
//		if(parentProvider != null)
//			initSupport.couldInitialize(parentProvider);
	}
	/*
	public abstract Type getBeanGenericType(String iName) throws UnresolvedBeanError;

	public abstract Class getBeanType(String iName) throws UnresolvedBeanError;

	public abstract Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception;
	*/

	/**
	 * Has to be overloaded
	 * @throws UnresolvedBeanError 
	 */
	protected Type getParentBeanGenericType(String iName) throws UnresolvedBeanError
	{
		/*
		Type t = getLocalBeanGenericType(iName);
		if(t != null)
			return t;
		*/
		if(parentProvider != null){
			return parentProvider.getBeanGenericType(iName);
		}
		
	    return null;
	}
	/**
	 * Has to be overloaded
	 * @throws UnresolvedBeanError 
	 */
	protected Class<?> getParentBeanType(String iName) throws UnresolvedBeanError
	{
		/*
		Class c = getLocalBeanType(iName);
		if(c != null)
			return c;
		*/
		if(parentProvider != null){
			return parentProvider.getBeanType(iName);
		}
		
	    return null;
	}
	/**
	 * Retrieves the dynamic bean value from the request
	 */
	protected Object getParentBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		/*
		Object o = getLocalBeanValue(iRequest, iName);
		if(t != null)
			return t;
		*/
		if(parentProvider != null){
			return parentProvider.getBeanValue(iRequest, iName);
		}
		
	    return null;
	}
	/**
	 * Retrieves the dynamic bean value from the request
	 */
	protected List<String> getParentBeanNames()
	{
		/*
		Object o = getLocalBeanValue(iRequest, iName);
		if(t != null)
			return t;
		*/
		if(parentProvider != null){
			return parentProvider.getBeanNames();
		}
		
	    return new ArrayList<String>();
	}
	protected String getParentBeanDescription(String name)
	{
		if(parentProvider != null){
			return parentProvider.getBeanDescription(name);
		}
		return null;
	}
}
