/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.ow2.opensuit.xmlmap.test;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;


import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.ow2.opensuit.xmlmap.DocGenerator;
import org.ow2.opensuit.xmlmap.schema.ISchema;
import org.ow2.opensuit.xmlmap.schema.SchemasManager;
import org.w3c.dom.Document;

/**
 * This test case implements some XMLMap integration tests:
 *  - build (Open SUIT) schema
 *  - generate XSD
 *  - generate XML doc
 *  
 * @author Adrien Ruffie / Pierre Smeyers
 * @author http://opensuit.ow2.org/
 * @version 1.0 Allow to test the <XsdGenerator> class
 */
public class XMLMapTestCase
{
	private SchemasManager schemas;
    /**
     * Allow to recover arguments for main() invocation of <XsdGenerator>, from
     * test.properties located into src/test/resources/test.properties
     * 
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception {
		schemas = new SchemasManager();
    }
    
    /**
     * Allow to test the xsd generation from opensuit core data structure
     * @throws Exception 
     */
    @Test
    public void testBuildXsd() throws Exception
    {
		// --- build main schema from source code
    	ISchema osuitSchema = schemas.declareSchema("base", "base.xsd", "org.ow2.opensuit.xml.base");
    	osuitSchema.buildFromClassPath();
    	
		// --- generate XSD DOM
		DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document xmlDoc = docBuilder.newDocument();
		osuitSchema.generateXSD(xmlDoc);
		
		// --- test the generated XSD with some xpath expressions
		// TODO
    }
    /**
     * Allow to test the doc generation from opensuit core
     * xml base data structure
     * @throws Exception 
     */
    @Test
    public void testBuildDoc() throws Exception 
    {
		// --- build main schema from source code
    	ISchema osuitSchema = schemas.declareSchema("base", "base.xsd", "org.ow2.opensuit.xml.base");
    	osuitSchema.buildFromClassPath();
    	
    	File docTargetDir = new File("target/test/xmldoc");
    	
    	// --- generate xmldoc
		DocGenerator generator = new DocGenerator(schemas);
		generator.buildDoc(osuitSchema , docTargetDir);
		
		// --- test the generated doc
		File index = new File(docTargetDir, "org.ow2.opensuit.xml.base_index.html");
		Assert.assertTrue("Index html file not created.", index.exists());
    }

}
